*     Main DCTEST
*     this is the driver module for the dctest program.
*
      implicit none
*     
*     If your machine is short on memory, you may wish to change the value of 
*     'ld', which dimensions all of the spaces, work or otherwise.  You will
*     also need to alter the input file to use matrices of appropriate sizes.
*
*     ITPARAM influences the number of iterations used to determine the
*     actual run-time for small-moderate sized matrices.  The formula is:
*     iterations = ( ITPARAM / N ) ** 2  in integer arithmetic.
*
      integer ld, itparam, maxdim, maxmode
      parameter (ld=1100, itparam=333, maxdim = 24, maxmode=6)
*     
      integer info, i, j, k, l, iseed(4), n, iter
      integer iwork(10*ld)
      integer iblock(ld),isplit(ld),ifail(ld),
     $     dims(maxdim), modes(maxmode), numdim, nummode
      logical dense, tridiag, special, lsame
      real d(ld), e(ld), work(ld,5*ld+3), 
     $     a(ld,ld), worka(ld,ld), tau(ld),
     $     workd(ld), worke(ld), q(ld,ld),
     $     slamacc(ld)
*     
      real one, two, three, four, zero
      parameter (one=1.0e0, two=2.0e0,three=3.0e0,
     $     four=4.0e0, zero=0.0e0)
*     
      intrinsic char, ichar
*     
      real second, eps, riter, cond
*     
      real slamch, slaran, slange, snrm2
      external slaran, second, errorchkg, sstedc, sstebz,
     $     sstein, ssteqr, ssytrd, sorgtr, slange, sgemm,
     $     slamch, snrm2, lsame, slatms
      
      data iseed/2156, 833, 2173,2373/
      eps = slamch( 'Eps' )
*     
      open(50, file='input', status='old')
      read(50, *) numdim
      do 20  i = 1, numdim
         read(50, *) dims( i )
 20   continue
      read(50, *) nummode
      do 30  i = 1, nummode
         read(50, *) modes( i )
 30   continue
      read(50, *) dense
      read(50, *) tridiag
      read(50, *) special
c      go to 199
*
*    Random Matrices 
*
      if ( dense ) then
	 call denopen( 'den' )
      end if
      if ( tridiag ) then
	 call triopen( 'tri' )
      end if
*     
      if ( dense .or. tridiag ) then
         write(06,*) 'random matrices (uniform on (-1,1))'
         do 100  l = 1, numdim
            n = dims( l )
*     
            iter = (ITPARAM / n)**2
            if ( iter .lt. 1 ) iter = 1
            riter = real( iter )
*     
            if (dense) then
*     
*     Create symmetric matrix A
*
               write(06,*) ' dense,  N  = ', n
               do 110 j=1, n
                  do 110 i=1, n
                     if (i.ge.j) then
                        a(i,j)=two*slaran(iseed)-one
                     else
                        a(i,j)=a(j,i)
                     end if
 110           continue
               call dentest( itparam, n, a, ld, d, e, q, 
     $              workd, worke, work, tau, slamacc, worka, 
     $              iwork, iblock, isplit, ifail, 
     $              123, 124, 136, 
     $              120, 130, 131, 
     $              121, 132, 133, 
     $              122, 134, 135 )
            end if
            if (tridiag) then
               write(06,*) ' tridiag,  N  = ', n
*     
*     Create random symmetric tridiagonal matrices
*        
*        simulate tridiag reduced from random dense
               do 120  i=1,n
                  D(i)=two*slaran(iseed)-one
                  e(i)=zero
                  do 130  j = n-1-i, 1, -1
                     e(i)=e(i)+(slaran(iseed))**2
 130              continue
                  e(i)=sign(sqrt(e(i)), (two*slaran(iseed)-one) )
     $                 + two*slaran(iseed)-one
 120           continue
*        random tridiagonal
c           do 210 i=1,n
c              D(i)=two*slaran(iseed)-one
c              e(i)=two*slaran(iseed)-one
c210        continue
               call tritest( itparam, n, d, e, q, ld, 
     $              workd, worke, work, tau, slamacc, 
     $              iwork, iblock, isplit, ifail, 
     $              220, 230, 231, 
     $              221, 232, 233, 
     $              222, 234, 235 )
            end if
 100     continue
      end if
*     
      if ( dense ) then
	 call denclose()
      end if
      if ( tridiag ) then
	 call triclose()
      end if
*
*     SLATMS matrices
*
 199  continue
      if ( nummode .gt. 0 .and. ( dense .or. tridiag ) ) then
         do 200  i = 1, nummode
            write(06,*) 'matrices from SLATMS, mode = ', modes(i)
            if ( dense ) then
	       call denopen( 'm'//char(ichar('0')+modes(i))//'d' )
            end if
            if (tridiag) then
	       call triopen( 'm'//char(ichar('0')+ modes(i))//'t' )
            end if
            cond = one / EPS
            do 210  k = 1, numdim
               n = dims( k )
*     dense
               if (dense) then
                  write(06,*) ' dense, N = ', N
                  call slatms( n, n, 's', iseed, 's', d, modes( i ), 
     $                 cond, one, n, n, 'n', a, ld, work, info )
                  call dentest( itparam, n, a, ld, d, e, q, 
     $                 workd, worke, work, tau, slamacc, worka, 
     $                 iwork, iblock, isplit, ifail, 
     $                 123, 124, 136, 
     $                 120, 130, 131, 
     $                 121, 132, 133, 
     $                 122, 134, 135 )
               end if 
*     tridiagonal
               if (tridiag) then
                  write(06,*) ' tridiag, N = ', N
                  call slatms( n, n, 's', iseed, 's', d, modes( i ), 
     $                 cond, one, 1, 1, 'n', a, ld, work, info )
                  call scopy( n, a(1,1), ld+1, d, 1 )
                  call scopy( n-1, a(1,2), ld+1, e, 1 )
                  call tritest( itparam, n, d, e, q, ld, 
     $                 workd, worke, work, tau, slamacc, 
     $                 iwork, iblock, isplit, ifail, 
     $                 220, 230, 231, 
     $                 221, 232, 233, 
     $                 222, 234, 235 )
               end if
 210        continue
            if ( dense ) then
               call denclose()
            end if
            if ( tridiag ) then
               call triclose()
            end if
 200     continue
      end if
*
****************************************************************
*     Special Matrices
*
      if (special) then
*
*     10 glued Wilkinson Matrices (glue = 1e-4)
*
         n = 10 * 21
         if ( ld .ge. n ) then
            write(06,*) '10 glued (1e-4) Wilkinson matrices'
            call triopen( 'wil' )
            do 300  i = 1, 10
               do 310 j = 1, 21
                  d( (i-1)*21 + j ) = abs( 11 - j )
                  e( (i-1)*21 + j ) = 1
 310           continue
               e( i*21 ) = 1.0e-4
 300        continue
            call tritest( itparam, n, d, e, q, ld, 
     $           workd, worke, work, tau, slamacc, 
     $           iwork, iblock, isplit, ifail, 
     $           220, 230, 231, 
     $           221, 232, 233, 
     $           222, 234, 235 )
            call triclose()
         end if
*     
*     121 matrix for n = 100:100:500
*     
         if ( ld .ge. 500 ) then
            write(06,*) '121 matrices'
            call triopen( '121' )
            do 400  n = 100, 500, 100
               write(6,*) 'N = ', n
               do 410  i = 1, n
                  d( i ) = 2
                  e( i ) = 1
 410           continue
               call tritest( itparam, n, d, e, q, ld, 
     $              workd, worke, work, tau, slamacc, 
     $              iwork, iblock, isplit, ifail, 
     $              220, 230, 231, 
     $              221, 232, 233, 
     $              222, 234, 235 )
 400        continue
            call triclose()
         end if
*     
*     BCSST08
*     
         open(60, file='BCSST08.dat', status='old')
         read(60, *) n
	 if ( ld .lt. n ) then
	    close(60)
         else
            do 500  i = 1,n
               read(60,*) d(i)
 500        continue
            read(60,*) e(n)
            do 510  i = 1,n-1
               read(60,*) e(i)
 510        continue
            close( 60 )
            write(06,*) 'BCSST08 matrix, N  = ', n
	    call triopen( 'BCS' )
*     solve system
*     check residuals
            call tritest( itparam, n, d, e, q, ld, 
     $           workd, worke, work, tau, slamacc, 
     $           iwork, iblock, isplit, ifail, 
     $           220, 230, 231, 
     $           221, 232, 233, 
     $           222, 234, 235 )
   	    call triclose()
         end if
*     
*     LUND...
*     
         open(60, file='LUND.dat', status='old')
         read(60, *) n
	 if ( ld .lt. n ) then
	    close(60)
         else
            do 600  i = 1,n
               read(60,*) d(i)
 600        continue
            read(60,*) e(n)
            do 610  i = 1,n-1
               read(60,*) e(i)
 610        continue
            close( 60 )
            write(06,*) 'LUND matrix, N  = ', n
            call triopen( 'LUN' )
*     solve system
*     check residuals
            call tritest( itparam, n, d, e, q, ld, 
     $           workd, worke, work, tau, slamacc, 
     $           iwork, iblock, isplit, ifail, 
     $           220, 230, 231, 
     $           221, 232, 233, 
     $           222, 234, 235 )
            call triclose()
         end if
*     
      end if
*     
*************************************************************************
         
*
      stop
      end
*
*
*     Dense
*
      subroutine dentest( itparam, n, a, ld, d, e, q, 
     $     workd, worke, work, tau, slamacc, worka, 
     $     iwork, iblock, isplit, ifail, 
     $     mmtfile, trtfile, trqefile,
     $     dctfile, dcdefile, dcqefile, 
     $     qrtfile, qrdefile, qrqefile, 
     $     bstfile, bsdefile, bsqefile )
*     
      implicit none
*     
      integer itparam, n, ld,
     $     mmtfile, trtfile, trqefile,
     $     dctfile, dcdefile, dcqefile, 
     $     bstfile, bsdefile, bsqefile, 
     $     qrtfile, qrdefile, qrqefile
      
      real a( ld, * )
*     
      integer info, i, k, iseed(4), iter 
*     
      integer iwork(*)
      integer nsplit,il,iu,iblock(*),isplit(*),ifail(*),m
      real work(ld,*), d( * ), e( * ),
     $     worka(ld,*), tau(*),
     $     workd(*), worke(*), q(ld,*),
     $     vl, vu, slamacc(*), 
     $     eps
*     
      real one, two, three, four, zero
      parameter (one=1.0e0, two=2.0e0,three=3.0e0,
     $     four=4.0e0, zero=0.0e0)
      
      real second, t, tdc, tmm, gtime, tqr, tbz, ttr,
     $     tin, tsetup, riter, tmcpy
      real slamch, slaran, slange, snrm2
      external slaran, second, errorchkg, sstedc, sstebz,
     $     sstein, ssteqr, ssytrd, sorgtr, slange, sgemm,
     $     slamch, snrm2, lsame
      
      data iseed/2156, 833, 2173,2373/
      eps = slamch( 'Eps' )
*     
      iter = (ITPARAM / n)**2
      if ( iter .lt. 1 ) iter = 1
      riter = real( iter )
*     
*     
*     Tridiagonalize the matrix
*     
      t=second(gtime)
      do 50  k = 1, iter
         call SLACPY( 'A', n, N, a, Ld, worka, LD )
 50   continue
      tmcpy = second(gtime) - t
*     Form the orthogonal matrix which tridiagonalizes A
c     write(06,*), 'Reduction to tridiagonal form.'
      do 60  k = 1, iter
         call SLACPY( 'A', n, N, a, Ld, worka, LD )
         call SSYTRD( 'L', N, worka, LD, D, E, TAU,
     $        WORK, n*n, INFO )
         call SORGTR( 'L', N, worka, LD, TAU, WORK,
     $        n*n, INFO )
 60   continue
      ttr = ( second(gtime)-t - tmcpy ) / riter
      write(trtfile, *) n, ttr
*     
      call sgemm( 't', 'n', n, n, n, one, worka, ld, 
     $     worka, ld, zero, work, ld )
c     do 55 i=1,n
c     work(i,i)=work(i,i)-one
c     55            continue
c     t = slange( '1', n, n, work, n, work(1,n+1) )
c     t = t / eps
      t=zero
      do 65 i=1,n
         work(i,i)=work(i,i)-one
         t= max( t, snrm2(n,work(1,i),1) )
 65   continue
      t = t / eps
c     write *, 'Orthog. residual of A->T matrix is ',t
      write(trqefile, *) n, t
*     
*     D&C Part
*     
c     write(06,*) ' '
c     write(06,*) ' By SSTEDC'
c     write(06,*) ' '
      t=second(gtime)
      do 110  k = 1, iter
         call scopy( n, d, 1, workd, 1 )
         call scopy( n-1, e, 1, worke, 1 )
         call SLACPY( 'A', n, N, worka, LD, q, LD )
 110  continue
      tsetup = second(gtime) - t
*     
      t=second(gtime)
      info = 0
      do 130  k = 1, iter
         call scopy( n, d, 1, workd, 1 )
         call scopy( n-1, e, 1, worke, 1 )
         call SLACPY( 'A', n, N, worka, LD, q, LD )
         call sstedc('i', n,workd,worke,q,ld,iwork,work,info)
 130  continue
      tdc = ( second(gtime)-t - tsetup ) / riter
*     
      call SLACPY( 'A', n, N, q, LD, work, LD )
      T = SECOND( GTIME )
      do 140  k = 1, iter
         call sgemm('n','n',n,n,n,one,worka,ld,work,ld,
     $        zero,q,ld)
 140  continue
      tmm = ( second(gtime)-t ) / riter
      write(mmtfile, *) n, tmm
      write(dctfile, *) n, ttr + tdc + tmm
*     Check accuracy
      call errorchkg(n, a, workd,q,ld,work)
      write(dcdefile, *) n, work(1,1)
      write(dcqefile, *) n, work(2,1)
      call ssbt21( 'U', n, n-1, 0, a, ld, workd, worke, 
     $     q, ld, work(1,2), work)
      write(dcdefile, *) n, work(1,1)
      write(dcqefile, *) n, work(2,1)
*     
*     QR Part
*     
c     write(06,*) ' '
c     write(06,*) ' By SSTEQR'
c     write(06,*) ' '
*     Timing SSTEQR for solving eigesystem of tridiagonal matrices.
      T = SECOND( GTIME )
      do 150  k = 1, iter
         call scopy( n, d, 1, workd, 1 )
         call scopy( n-1, e, 1, worke, 1 )
         call SLACPY( 'A', n, N, worka, LD, q, LD )
         CALL SSTEQR( 'v', N, workD, workE,Q, LD, work,INFO)
 150  continue
      Tqr = ( SECOND( GTIME ) - T - tsetup ) / riter
      write(qrtfile, *) n, ttr + tqr
*     Check accuracy
      call errorchkg(n, a, workd,q,ld,work)
      write(qrdefile, *) n, work(1,1)
      write(qrqefile, *) n, work(2,1)
      call ssbt21( 'U', n, n-1, 0, a, ld, workd, worke, 
     $     q, ld, work(1,2), work)
      write(qrdefile, *) n, work(1,1)
      write(qrqefile, *) n, work(2,1)
*     
*     BS Part
*     
c     write(06,*) ' '
c     write(06,*) ' By SSTEBZ with SSTEIN '
c     write(06,*) ' '
      T = SECOND( GTIME )
      do 160  k = 1, iter
         call scopy( n, d, 1, workd, 1 )
         call scopy( n-1, e, 1, worke, 1 )
         call SLACPY( 'A', n, N, worka, LD, q, LD )
         call sstebz( 'A','B',N,vl,vu,il,iu,0.0,workd,worke,
     $        m,nsplit,slamacc,iblock,isplit,work,iwork,
     $        info )
 160  continue
      tbz = ( SECOND( GTIME ) - T - tsetup ) / riter
*     
      call SLACPY( 'A', n, N, worka, LD, q, LD )
*     
      T = SECOND( GTIME )
      do 180  k = 1, iter
         call SLACPY( 'A', n, N, worka, LD, q, LD )
         call sstein(n,workd,worke,m,slamacc,iblock,isplit,
     $        q,ld,work,iwork,ifail,info )
 180  continue
      Tin = ( SECOND( GTIME ) - T - tmcpy ) / riter
*     
      call SLACPY( 'A', n, N, q, LD, work, LD )
*     
      call sgemm('n', 'n', n,n,n,one,worka,ld,work,ld,
     $     zero,q,ld)
      write(bstfile, *) n, ttr + tbz + tin + tmm
*     Check accuracy
      call errorchkg(n, a, slamacc,q,ld,work)
      write(bsdefile, *) n, work(1,1)
      write(bsqefile, *) n, work(2,1)
      call ssbt21( 'U', n, n-1, 0, a, ld, slamacc, worke, 
     $     q, ld, work(1,2), work)
      write(bsdefile, *) n, work(1,1)
      write(bsqefile, *) n, work(2,1)
*     
      return
*     
      end
*     
*     
*     Tridiagonal
*     
      subroutine tritest( itparam, n, d, e, q, ld, 
     $     workd, worke, work, tau, slamacc, 
     $     iwork, iblock, isplit, ifail, 
     $     dctfile, dcdefile, dcqefile, 
     $     qrtfile, qrdefile, qrqefile, 
     $     bstfile, bsdefile, bsqefile )
*     
      implicit none
*     
      integer itparam, n, iwork( * ), ld, 
     $     iblock(*),isplit(*),ifail(*), 
     $     dctfile, dcdefile, dcqefile, 
     $     bstfile, bsdefile, bsqefile, 
     $     qrtfile, qrdefile, qrqefile
      
      real d( * ), e( * ), work(ld,*), tau(*), workd(*), worke(*), 
     $     q(ld,*), slamacc(*)
*     
      integer info, k, iseed(4), iter
*     
      integer nsplit,il,iu, m
      real vl, vu, eps
*     
      real one, two, three, four, zero
      parameter (one=1.0e0, two=2.0e0,three=3.0e0,
     $     four=4.0e0, zero=0.0e0)
 
      real second, t, tdc, gtime, tqr, tbz, 
     $     tin, tsetup, riter
      real slamch, slaran, slange, snrm2
      external slaran, second, errorchkg, sstedc, sstebz,
     $     sstein, ssteqr, ssytrd, sorgtr, slange, sgemm,
     $     slamch, snrm2, lsame

      data iseed/2156, 833, 2173,2373/
      eps = slamch( 'Eps' )
*     
      iter = (ITPARAM / n)**2
      if ( iter .lt. 1 ) iter = 1
      riter = real( iter )
*     
      t=second(gtime)
      do 510  k = 1, iter
         call scopy( n, d, 1, workd, 1)
         call scopy( n-1, e, 1, worke, 1)
 510  continue
      tsetup = second(gtime) - t
*     
*     D&C Part
*     
c     write(06,*) ' '
c     write(06,*) ' By SSTEDC'
c     write(06,*) ' '
      t=second(gtime)
      info = 0
      do 530  k = 1, iter
         call scopy( n, d, 1, workd, 1)
         call scopy( n-1, e, 1, worke, 1)
         call sstedc('i', n,workd,worke,q,ld,iwork,work,info)
 530  continue
      tdc = ( second(gtime) - t - tsetup ) / riter
      write(dctfile, *) n, tdc
*     Check accuracy
      call errorchk(n, d, e, workd,q,ld,work)
      write(dcdefile, *) n, work(1,1)
      write(dcqefile, *) n, work(2,1)
      call sstt21(n, 0, d, e, workd, worke, q, ld, work(1,2), work)
      write(dcdefile, *) n, work(1,1)
      write(dcqefile, *) n, work(2,1)
*     
*     QR Part
*     
c     write(06,*) ' '
c     write(06,*) ' By SSTEQR'
c     write(06,*) ' '
*     Timing SSTEQR for solving eigesystem of tridiagonal matrices.
      T = SECOND( GTIME )
      do 540  k = 1, iter
         call scopy( n, d, 1, workd, 1)
         call scopy( n-1, e, 1, worke, 1)
         CALL SSTEQR( 'i', N, workD, workE,Q, LD, work,INFO)
 540  continue
      tqr = ( SECOND( GTIME ) - T - tsetup ) / riter
      write(qrtfile, *) n, tqr
*     Check accuracy
      call errorchk(n, d, e, workd,q,ld,work)
      write(qrdefile, *) n, work(1,1)
      write(qrqefile, *) n, work(2,1)
      call sstt21(n, 0, d, e, workd, worke, q, ld, work(1,2), work)
      write(qrdefile, *) n, work(1,1)
      write(qrqefile, *) n, work(2,1)
*     
*     BS Part
*     
c     write(06,*) ' '
c     write(06,*) ' By SSTEBZ with SSTEIN '
c     write(06,*) ' '
      T = SECOND( GTIME )
      do 550  k = 1, iter
         call scopy( n, d, 1, workd, 1)
         call scopy( n-1, e, 1, worke, 1)
         call sstebz( 'A','B',N,vl,vu,il,iu,0.0,workd,worke,
     $        m,nsplit,slamacc,iblock,isplit,work,iwork,
     $        info )
 550  continue
      tbz = ( SECOND( GTIME ) - T - tsetup ) / iter
      T = SECOND( GTIME )
      call sstein(n,workd,worke,m,slamacc,iblock,isplit,q,ld,
     $     work,iwork,ifail,info )
      Tin=SECOND( GTIME ) - T
      write(bstfile, *) n, tbz + tin
*     Check accuracy
      call errorchk(n, d, e, slamacc,q,ld,work)
      write(bsdefile, *) n, work(1,1)
      write(bsqefile, *) n, work(2,1)
      call sstt21(n, 0, d, e, slamacc, worke, q, ld, work(1,2), work)
      write(bsdefile, *) n, work(1,1)
      write(bsqefile, *) n, work(2,1)
*     
      return
*     
      end
*     
      subroutine denopen( id )

      character*3 id
      character*11 filename

      filename = 'dc'//id//'.times'
      open(120, file=filename,status='new')
      filename = 'qr'//id//'.times'
      open(121, file=filename,status='new')
      filename = 'bs'//id//'.times'
      open(122, file=filename,status='new')
      filename = 'mm'//id//'.times'
      open(123, file=filename,status='new')
      filename = 'tr'//id//'.times'
      open(124, file=filename,status='new')
*
      filename = 'dc'//id//'d.errs'
      open(130, file=filename,status='new')
      filename = 'dc'//id//'q.errs'
      open(131, file=filename,status='new')
      filename = 'qr'//id//'d.errs'
      open(132, file=filename,status='new')
      filename = 'qr'//id//'q.errs'
      open(133, file=filename,status='new')
      filename = 'bs'//id//'d.errs'
      open(134, file=filename,status='new')
      filename = 'bs'//id//'q.errs'
      open(135, file=filename,status='new')
      filename = 'tr'//id//'q.errs'
      open(136, file=filename,status='new')
*
      return
      end
*
      subroutine denclose(  )
*
      close(120)
      close(121)
      close(122)
      close(123)
      close(124)
*
      close(130)
      close(131)
      close(132)
      close(133)
      close(134)
      close(135)
      close(136)
*
      return
      end
*
      subroutine triopen( id )

      character*3 id
      character*11 filename

      filename = 'dc'//id//'.times'
      open(220, file=filename,status='new')
      filename = 'qr'//id//'.times'
      open(221, file=filename,status='new')
      filename = 'bs'//id//'.times'
      open(222, file=filename,status='new')
*
      filename = 'dc'//id//'d.errs'
      open(230, file=filename,status='new')
      filename = 'dc'//id//'q.errs'
      open(231, file=filename,status='new')
      filename = 'qr'//id//'d.errs'
      open(232, file=filename,status='new')
      filename = 'qr'//id//'q.errs'
      open(233, file=filename,status='new')
      filename = 'bs'//id//'d.errs'
      open(234, file=filename,status='new')
      filename = 'bs'//id//'q.errs'
      open(235, file=filename,status='new')

      return
      end
*
      subroutine triclose(  )

      close(220)
      close(221)
      close(222)
*
      close(230)
      close(231)
      close(232)
      close(233)
      close(234)
      close(235)
*
      return
      end
