      SUBROUTINE errorchk( N, D, E, Dlam,Q,LDQ,WORK)
*    $                    evares, evtres)
*
*   -- This routine is designed to check the accuracy of
*      eigensystems of a tridiagnal matrix obtained in some
*      way.
*                                April 3, 1992
*
	  INTEGER           LDQ, N
*     ..Array Arguments ..
	  REAL   D( * ), E( * ), WORK( * ), Q( LDQ, * )
	  real   dlam(*), evares, evtres
*
*   Arguments
*  =========
*..
*  N       (input) INTEGER
*          The number of rows and columns in the matrix.  N >= 0.
*
*  D       (input) REAL array, dimension (N)
*          On entry, D contains the diagonal elements of the
*          tridiagonal matrix. It is unchanged on exit.
*
*  E       (input/output) REAL array, dimension (N)
*          On entry, E contains the subdiagonal elements of the
*          tridiagonal matrix in positions 1 through N-1.
*          It is unchanged on exit.
*
*  Dlam    (input)  REAL array, dimension (N)
*          On entry, DLAM contains the computed eigenvalues.
*
*  Q       (input) REAL array, dimension (LDZ, N)
*          Z contains the eigenvectors associated with the
*          stored eigenvalues.It is unchanged on exit.
*
*  LDQ     (input) INTEGER
*          The leading dimension of the array Q. 
*
*  evares  (output) REAL scalar.
*          It is equal to
*
*              || Aqi - dlam(i)*qi|| 
*          max --------------------- over 1 le i le N
*                 EPS * || A ||
*
*          for determining how accuate the computed eigensystems
*          are, where A is the orginal tri-diagonal matrix.
*
*  evtres  (output) REAL scalar.
*          It is equal to
*
*              || Q^T qi - ei|| 
*          max ----------------  over 1 le i le N
*                     EPS
*          for determining how accuate the computed eigensystems
*          are.
*
*  =====================================================================
*
*  EPS     is machine precision
*
*     .. local variables
	  REAL anorm, EPS
          INTEGER I, J, K
*
*     .. Parameters ..
	  REAL   ZERO, ONE, TWO
      PARAMETER          ( ZERO = 0.0E0, ONE = 1.0E0, TWO = 2.0E0 )
*     ..
*     .. External Functions ..
      REAL  SLAMCH,snrm2
      EXTERNAL           SLAMCH, snrm2
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX
*     ..
*     .. Executable Statements ..
*
      EPS = SLAMCH( 'Epsilon' )
*
*
*     Compute evares
*
*
*     Quick return if possible
*
	  IF( N.EQ.0 )
     $   RETURN
*
      IF( N.EQ.1 ) THEN
         evares=abs( d(1)-dlam(1) )/abs(eps*dlam(1))
	 evtres=abs( q(1,1)*q(1,1)-one )/eps
c         write(6,*) 'Residual Error in Computed Eigensystem  is  ',
c     $        evares
c         write(6,*) 'Error for Orth. Test in  Comp. Vectors  is  ',
c     $        evtres
         work(1) = evares
         work(2) = evtres
	 return
      end if
*
*     Compute the 2-norm of A.
*
      anorm = abs(dlam(1))
      do 10 j = 2, n
         anorm = max( abs(dlam(j)), anorm)
  10  continue
*
      evares=zero
      if (n.eq.2) then
	     Do 100 i=1,n
		work(1)=( d(1)-dlam(i) )*q(1,i)+e(1)*q(2,i)
		work(2)=e(1)*q(1,i)+( d(2)-dlam(i) )*q(2,i)
                evares=max( evares, snrm2(n,work,1) )
 100         continue
	  else
             do 200 i=1,n
		work(1)=( d(1)-dlam(i) )*q(1,i)+e(1)*q(2,i)
		work(n)=e(n-1)*q(n-1,i)+( d(n)-dlam(i) )*q(n,i)
		do 150 k=2,n-1
		   work(k)=e(k-1)*q(k-1,i)+( d(k)-dlam(i) )*q(k,i)
     $                   +e(k)*q(k+1,i)
 150            continue
                evares=max( evares, snrm2(n,work,1) )
 200          continue
	  end if
	  evares=evares/(anorm * eps)
*
*     Compute evtres
*
	  evtres=zero
      do 300 i=1,n
	 do 250 k=1,n
	    work(k)=zero
            do 230 j=1,n
                 work(k)=work(k)+q(j,k)*q(j,i)
 230        continue
 250     continue
	 work(i)=work(i)-one
         evtres= max( evtres, snrm2(n,work,1) )
 300  continue
      evtres = evtres/eps
*
c      write(6,*) 'Residual Error in Computed Eigensystem  is  ',evares
c      write(6,*) 'Error for Orth. Test in  Comp. Vectors  is  ',evtres
      work(1) = evares
      work(2) = evtres
*
*   End of errorchk
*
	  return
	  end
