      SUBROUTINE errorchkg( N, a, Dlam,Q,LD,WORK)
*
*   -- This routine is designed to check the accuracy of
*      eigensystems of a tridiagnal matrix obtained in some
*      way.
*                                April 3, 1992
*
          implicit none
	  INTEGER           LD, N
*     ..Array Arguments ..
	  REAL   A(LD,*), WORK(ld,* ), Q( LD, * )
	  real   dlam(*), evares, evtres
*
*   Arguments
*  =========
*..
*  N       (input) INTEGER
*          The number of rows and columns in the matrix.  N >= 0.
*
*  A       (input) REAL array, dimension (LD, N).
*          A(LD,N) is a symmetric matrix.
*
*  Dlam    (input)  REAL array, dimension (N)
*          On entry, DLAM contains the computed eigenvalues.
*
*  Q       (input) REAL array, dimension (LD, N)
*          Z contains the eigenvectors associated with the
*          stored eigenvalues.It is unchanged on exit.
*
*  LD     (input) INTEGER
*          The leading dimension of the array Q and A. 
*
*  WORK   (workspace) REAL array, dimension (LD, N).
*
*  evares  (output) REAL scalar.
*          It is equal to
*
*              || Aqi - dlam(i)*qi|| 
*          max --------------------- over 1 le i le N
*                 EPS * || A ||
*
*          for determining how accuate the computed eigensystems
*          are, where A is the orginal tri-diagonal matrix.
*
*  evtres  (output) REAL scalar.
*          It is equal to
*
*              || Q^T qi - ei|| 
*          max ----------------  over 1 le i le N
*                     EPS
*          for determining how accuate the computed eigensystems
*          are.
*
*  =====================================================================
*
*  EPS     is machine precision
*
*     .. local variables
	  REAL anorm, EPS
          integer          i, j
*
*     .. Parameters ..
	  REAL   ZERO, ONE, TWO
      PARAMETER          ( ZERO = 0.0E0, ONE = 1.0E0, TWO = 2.0E0 )
*     ..
*     .. External Functions ..
      REAL   SLAMCH, snrm2
      EXTERNAL           SLAMCH, snrm2, SGEMM
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX
*     ..
*     .. Executable Statements ..
*
      EPS = SLAMCH( 'Epsilon' )
*
*     Compute evares
*
*
*     Compute evares
*
*
*     Quick return if possible
*
	  IF( N.EQ.0 )
     $   RETURN
*
      IF( N.EQ.1 ) THEN
         evares=abs( a(1,1)-dlam(1) )/abs(eps*dlam(1))
	 evtres=abs( q(1,1)*q(1,1)-one )/eps
c         write(6,*) 'Residual Error in Computed Eigensystem  is  ',
c     $        evares
c         write(6,*) 'Error for Orth. Test in  Comp. Vectors  is  ',
c     $        evtres
         work(1,1) = evares
         work(2,1) = evtres
	 return
      end if
*
*     Compute the 2-norm of A.
*
      anorm = abs(dlam(1))
      do 10 j = 2, n
         anorm = max( abs(dlam(j)), anorm)
  10  continue
*
      do 30 j=1,n
        do 20 i=1,n 
           work(i,j)=dlam(j)*q(i,j)
 20     continue
 30   continue
      call sgemm('n', 'n', n,n,n,one,a,ld,q,ld,-one,work,ld)
      evares=zero
      do 40 i=1,n
         evares=max( evares, snrm2(n,work(1,i),1) )
 40   continue
      evares=evares/(anorm * eps)
*
*     Compute evtres
*
      call sgemm('t', 'n', n,n,n,one,q,ld,q,ld,zero,work,ld)
      do 50 i=1,n
         work(i,i)=work(i,i)-1
 50   continue
      evtres=zero
      do 60 i=1,n
         evtres= max( evtres, snrm2(n,work(1,i),1) )
 60   continue
      evtres = evtres/eps
*
c      write(6,*) 'Residual Error in Computed Eigensystem  is  ',evares
c      write(6,*) 'Error for Orth. Test in  Comp. Vectors  is  ',evtres
      work(1,1) = evares
      work(2,1) = evtres
*
*   End of errorchk
*
	  return
	  end
