      SUBROUTINE SLAED1( N, CUTPNT, D, Q, LDQ, RHO, INDXQ, 
     $                    IWORK, WORK, INFO )
*
*  -- LAPACK routine (version 2.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     January 15, 1994
*
*     .. Scalar Arguments ..
      INTEGER           N, CUTPNT, LDQ, INFO
      REAL              RHO
*     ..
*     .. Array Arguments ..
      INTEGER           IWORK( * ), INDXQ( * )
      REAL              D( * ), Q( LDQ, * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  SLAED1 computes the updated eigensystem of a symmetric tridiagonal 
*  matrix after modification by a rank-one symmetric matrix.  This 
*  routine is used only for the eigenproblem which requires all 
*  eigenvalues and eigenvectors of a tridiagonal matrix.  SLAED7 handles
*  the case in which eigenvalues only or eigenvalues and eigenvectors 
*  of a full symmetric matrix (which was reduced to tridiagonal form) 
*  are desired.
*
*    T = Q(in) ( D(in) + RHO * Z*Z' ) Q'(in) = Q(out) * D(out) * Q'(out)
*
*     where Z = Q'u, u is a vector of length N with ones in the
*     CUTPNT and CUTPNT + 1 th entries and zeros elsewhere.
*
*     The eigenvectors of the original matrix are stored in Q, and the 
*     eigenvalues are in D.  The algorithm consists of three stages:
*
*        The first stage consists of deflating the size of the problem 
*        when there are multiple eigenvalues or if there is a zero in 
*        the Z vector.  For each such occurence the dimension of the 
*        secular equation problem is reduced by one.  This stage is 
*        performed by the routine SLAED2.
*
*        The second stage consists of calculating the updated 
*        eigenvalues. This is done by finding the roots of the secular 
*        equation via the routine SLAED4 (as called by SLAED3).  
*        This routine also calculates the eigenvectors of the current 
*        problem.
*
*        The final stage consists of computing the updated eigenvectors
*        directly using the updated eigenvalues.  The eigenvectors for 
*        the current problem are multiplied with the eigenvectors from 
*        the overall problem.
*
*  Arguments
*  =========
*
*  N      (input) INTEGER
*         The dimension of the symmetric tridiagonal matrix.  N >= 0.
*
*  CUTPNT (input) INTEGER
*         Contains the location of the last eigenvalue in the leading
*         sub-matrix.  MIN(1,N) <= CUTPNT <= N.
*
*  D      (input/output) REAL array, dimension(N)
*         On entry D contains the eigenvalues of the rank-1-perturbed 
*           matrix.
*         On exit D contains the eigenvalues of the repaired matrix.
*
*  Q      (input/output) REAL array, dimension(LDQ,N)
*         On entry Q contains the eigenvectors of the rank-1-perturbed
*            matrix.
*         On exit Q contains the eigenvectors of the repaired 
*            tridiagonal matrix.
*
*  LDQ    (input) INTEGER
*         The leading dimension of the array Q.  LDQ >= max( 1, N ).
*
*  RHO    (input) REAL
*         Contains the subdiagonal entry used to create the rank-1
*         modification.
*
*  INDXQ  (output) INTEGER array, dimension(N)
*         This contains the permutation which will reintegrate the 
*         subproblem just solved back into sorted order, 
*         i.e. D( INDXQ( I = 1, N ) ) will be in ascending order.      
*
*  IWORK  (workspace) INTEGER array, dimension( 4 * N )
*
*  WORK  (workspace) REAL array, dimension( 3*N + 2*N**2 )
*
*  INFO   (output) INTEGER
*          = 0:  successful exit.
*          < 0:  if INFO = -i, the i-th argument had an illegal value.
*          > 0:  if INFO = 1, an eigenvalue did not converge
*
*  =====================================================================
*
*     .. Local Scalars ..
      INTEGER         I, IZ, IDLMDA, IW, IQ2, IS, INDX, INDXP,
     $                INDXC, COLTYP, K, CPP1, 
     $                N1, N2, IND1, IND2, LDQ2
*     ..
*     .. External Subroutines ..
      EXTERNAL        SCOPY, SLAED2, SLAED3, SLAMRG, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
      IF( N.LT.0 ) THEN
         INFO = -1
      ELSE IF ( MIN( 1, N ) .GT. CUTPNT  .OR.
     $          N .LT. CUTPNT ) THEN
         INFO = -2
      ELSE IF ( LDQ.LT.MAX( 1, N ) ) THEN
         INFO = -5
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'SLAED1', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF ( N .EQ. 0 ) 
     $     RETURN
*
*     The following values are for bookkeeping purposes only.  They are
*     integer pointers which indicate the portion of the workspace
*     used by a particular array in SLAED2 and SLAED3.
*
      LDQ2 = N
*
      IZ = 1
      IDLMDA = IZ + N
      IW = IDLMDA + N
      IQ2 = IW + N
      IS = IQ2 + N * LDQ2
*
      INDX = 1
      INDXC = INDX + N
      COLTYP = INDXC + N
      INDXP = COLTYP + N
      
*      
*     Form the z-vector which consists of the last row of Q_1 and the
*     first row of Q_2.
*
      CALL SCOPY( CUTPNT, Q(CUTPNT, 1), LDQ, WORK( IZ ), 1 )
      CPP1 = CUTPNT + 1
      CALL SCOPY( N - CUTPNT, Q(CPP1, CPP1), LDQ,
     $            WORK( IZ + CUTPNT ), 1 )
*
*     Deflate eigenvalues.
*
      CALL SLAED2( K, N, Q, LDQ, D, RHO, CUTPNT, WORK( IZ ), 
     $     WORK( IDLMDA ), WORK( IQ2 ), LDQ2, WORK( IW ), 
     $     IWORK( INDXP ), IWORK ( INDX ), IWORK( INDXC ), 
     $     IWORK( COLTYP ), INDXQ, INFO )
*
*     Solve Secular Equation.
*
      IF ( K .NE. 0 ) THEN
         CALL SLAED3( K, 1, K, N, Q, LDQ, D, RHO, 
     $        WORK( IDLMDA ), WORK( IQ2 ), LDQ2, WORK( IW ), 
     $        WORK( IS ), K, IWORK( INDXC ), CUTPNT, 
     $        IWORK( COLTYP ), INFO )
         IF ( INFO .NE. 0 ) THEN
            RETURN
         END IF
*
*     Prepare the INDXQ sorting premutation.
*
         N1 = K
         N2 = N - K
         IND1 = 1
         IND2 = N
         CALL SLAMRG( N1, N2, D, 1, -1, INDXQ )
      ELSE
         DO 111  I = 1, N
            INDXQ( I ) = I
 111     CONTINUE
      END IF
*
      RETURN
*
*     End of SLAED1
*
      END
