      SUBROUTINE  SLAED2( K, N, Q, LDQ, D, RHO, CUTPNT, Z, 
     $                    DLAMDA, Q2, LDQ2, W, INDXP, INDX, INDXC, 
     $                    COLTYP, INDXQ, INFO )
*
*  -- LAPACK routine (version 2.0) --
*     Univ. of Tennessee, Oak Ridge National Lab, Argonne National Lab,
*     Courant Institute, NAG Ltd., and Rice University
*     January 15, 1994
*
*     .. Scalar Arguments ..
      INTEGER            K, LDQ, LDQ2, N, CUTPNT, INFO
      REAL               RHO
*     ..
*     .. Array Arguments ..
      INTEGER            INDX( * ), INDXP( * ), COLTYP( * ), INDXC( * ),
     $                   INDXQ( * )
      REAL               D( * ), DLAMDA( * ), Q( LDQ, * ),
     $                   Q2( LDQ2, * ), W( * ), Z( * )
*     ..
*
*  Purpose
*  =======
*
*  SLAED2 merges the two sets of eigenvalues together into a single 
*  sorted set.  Then it tries to deflate the size of the problem.  
*  There are two ways in which deflation can occur:  when two or more 
*  eigenvalues are close together or if there is a tiny entry in the 
*  Z vector.  For each such occurrence the order of the related secular
*  equation problem is reduced by one.  
*
*  Arguments
*  =========
*
*  K      (output) INTEGER
*         Contains the number of non-deflated eigenvalues.
*         This is the order of the related secular equation. 0 <= K <=N.
*
*  N      (input) INTEGER
*         The dimension of the symmetric tridiagonal matrix.  N >= 0.
*
*  Q      (input/output) REAL array, dimension(LDQ,N)
*         On entry Q contains the eigenvectors of two submatrices in
*         the two square blocks with corners at (1,1), (CUTPNT,CUTPNT)
*         and (CUTPNT+1, CUTPNT+1), (N,N).
*         On exit Q contains the trailing (N-K) updated eigenvectors 
*         (those which were deflated) in its last N-K columns.
*
*  LDQ    (input) INTEGER
*         The leading dimension of the array Q.  LDQ >= max( 1, N ).
*
*  D      (input/output) REAL array, dimension(N)
*         On entry D contains the eigenvalues of the two submatrices to
*         be combined.  On exit D contains the trailing (N-K) updated 
*         eigenvalues (those which were deflated) sorted into increasing
*         order.
*
*  RHO    (input/output) REAL
*         Contains the off diagonal element associated with the rank-1 
*         cut which originally split the two submatrices which are now being 
*         recombined. RHO is modified during the computation to the 
*         value required by SLAED3.
*
*  CUTPNT (input) INTEGER
*         Contains the location of the last eigenvalue in the leading
*         sub-matrix.  MIN(1,N) <= CUTPNT <= N.
*
*  Z      (input) REAL array, dimension(N)
*         On input this vector contains the updating vector (the last 
*         row of the first sub-eigenvector matrix and the first row of 
*         the second sub-eigenvector matrix).  The contents of Z are 
*         destroyed during the updating process.
*
*  DLAMDA (output) REAL array, dimension(N)
*         Contains a copy of the first K eigenvalues which will be used
*         by SLAED3 to form the secular equation.
*
*  Q2     (output) REAL array, dimension(LDQ2,N)
*         Contains a copy of the first K eigenvectors which will be used
*         by SLAED3 in a matrix multiply (SGEMM) to solve for the new
*         eigenvectors.   Q2 is arranged into three blocks.  The  
*         first block contains non-zero entries only at and above 
*         CUTPNT, the second contains non-zero entries only below 
*         CUTPNT, and the third is dense.
*
*  LDQ2   (input) INTEGER
*         The leading dimension of the array Q2.  LDQ2 >= max( 1, N ).
*
*  W      (output) REAL array, dimension(N)
*         This will hold the first k values of the final 
*         deflation-altered z-vector and will be passed to SLAED3.
*
*  INDXP  (workspace) INTEGER array, dimension(N)
*         This will contain the permutation used to place deflated 
*         values of D at the end of the array. On output INDXP(1:K)
*         points to the nondeflated D-values and INDXP(K+1:N)
*         points to the deflated eigenvalues.
*
*  INDX   (workspace) INTEGER array, dimension(N)
*         This will contain the permutation used to sort the contents of
*         D into ascending order.
*
*  INDXC  (output) INTEGER array, dimension(N)
*         This will contain the permutation used to arrange the columns
*         of the deflated Q matrix into three groups:  the first group 
*         contains non-zero entries only at and above CUTPNT, the second
*         contains non-zero entries only below CUTPNT, and the third is
*         dense.
*
*  COLTYP (workspace) INTEGER array, dimension(N)
*         This will contain a label which will indicate which of the
*         following types a column in the Q2 matrix is:
*         1 : non-zero in the upper half only
*         2 : non-zero in the lower half only
*         3 : dense
*         4 : deflated
*
*  INDXQ  (input) INTEGER array, dimension(N)
*         This contains the permutation which separately sorts the two 
*         sub-problems in D into ascending order.  Note that entries in 
*         the second half of this permutation must first have CUTPNT
*         added to their values.
*
*  INFO   (output) INTEGER
*          = 0:  successful exit.
*          < 0:  if INFO = -i, the i-th argument had an illegal value.
*
*  =====================================================================
*
*     .. Parameters ..
      REAL               ZERO, ONE, MONE, EIGHT, TWO
      PARAMETER          ( MONE = -1.0E0, ZERO = 0.0E0, ONE = 1.0E0, 
     $                     TWO = 2.0E0, EIGHT = 8.0E0 )
*     ..
*     .. Local Arrays ..
      INTEGER            CTOT( 4 ), PSM( 4 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, IMAX, J, JPREV, JMAX, JP, K2, N1,
     $                   N1P1, N2, CT

      REAL               C, EPS, S, T, TAU, TOL
*     ..
*     .. External Functions ..
      INTEGER            ISAMAX
      REAL               SLAMCH, SLAPY2
      EXTERNAL           ISAMAX, SLAPY2, SLAMCH
*     ..
*     .. External Subroutines ..
      EXTERNAL           SCOPY, SLACPY, SLAMRG, SROT, SSCAL, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
      IF( N .LT. 0 ) THEN
         INFO = -2
      ELSE IF ( LDQ .LT. MAX( 1, N ) ) THEN
         INFO = -4
      ELSE IF ( MIN( 1, N ) .GT. CUTPNT  .OR.
     $          N .LT. CUTPNT ) THEN
         INFO = -7
      ELSE IF ( LDQ2 .LT. MAX( 1, N ) ) THEN
         INFO = -11
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'SLAED2', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF ( N .EQ. 0 ) 
     $     RETURN
*
      N1 = CUTPNT
      N2 = N - N1
      N1P1 = N1 + 1
*
      IF ( RHO .LT. ZERO ) THEN
         CALL SSCAL( N2, MONE, Z( N1P1 ), 1 )
      END IF
*
*     Normalize z so that norm(z) = 1.  Since z is the concatenation of
*     two normalized vectors, norm2(z) = sqrt(2).
*
      T = ONE / SQRT( TWO )
      DO 10 J = 1, N
         INDX( J ) = J
   10 CONTINUE
      CALL SSCAL( N, T, Z, 1 )
*     RHO = ABS( norm(z)**2 * RHO )
      RHO = ABS( TWO * RHO )
*
      DO 15 I = 1, CUTPNT
         COLTYP(I) = 1
 15   CONTINUE
      DO 25 I = CUTPNT + 1, N
         COLTYP(I) = 2
 25   CONTINUE
*
*     Sort the eigenvalues into increasing order
*
      DO 35  I = CUTPNT+1, N
         INDXQ( I ) = INDXQ( I ) + CUTPNT
 35   CONTINUE
*     re-integrate the deflated parts from the last pass
      DO 45  I = 1, N
         DLAMDA( I ) = D( INDXQ( I ) )
         W( I ) = Z( INDXQ( I ) )
         INDXC( I ) = COLTYP( INDXQ( I ) )
 45   CONTINUE
      CALL SLAMRG( N1, N2, DLAMDA, 1, 1, INDX )
      DO 55  I = 1, N
         D( I ) = DLAMDA( INDX( I ) )
         Z( I ) = W( INDX( I ) )
         COLTYP( I ) = INDXC( INDX( I ) )
 55   CONTINUE
*
*     Calculate the allowable deflation tolerence
*
      IMAX = ISAMAX( N, Z, 1 )
      JMAX = ISAMAX( N, D, 1 )
      EPS = SLAMCH( 'Epsilon' )
      TOL = EIGHT * EPS * ABS( D( JMAX ) )
*
*     If the rank-1 modifier is small enough, no more needs to be done --
*     except to reorganize Q so that its columns correspond with the 
*     entries in D.
*
      IF( RHO*ABS( Z( IMAX ) ).LE.TOL ) THEN
         K = 0
         DO 75  J = 1, N
            CALL SCOPY( N, Q( 1, INDXQ(INDX( J ))), 1, 
     $           Q2( 1, J ), 1 )
 75      CONTINUE
         CALL SLACPY( 'A', N, N, Q2, LDQ2, Q, LDQ )
         RETURN
      END IF
*
*     If there are multiple eigenvalues then the problem deflates.  Here the 
*     number of equal eigenvalues are found.  As each equal eigenvalue is
*     found, an elementary reflector is computed to rotate the
*     corresponding eigensubspace so that the corresponding components of
*     Z are zero in this new basis.
*
      K = 0
      K2 = N + 1
      DO 50 J = 1, N
         IF( RHO*ABS( Z( J ) ).LE.TOL ) THEN
*
*           Deflate due to small z component.
*
            K2 = K2 - 1
            INDXP( K2 ) = J
            COLTYP( J ) = 4
            IF( J.EQ.N )
     $         GO TO 90
         ELSE
            JPREV = J
            GO TO 60
         END IF
   50 CONTINUE
   60 CONTINUE
      J = J + 1
      IF( J.GT.N )
     $   GO TO 80
      IF( RHO*ABS( Z( J ) ).LE.TOL ) THEN
*
*        Deflate due to small z component.
*
         K2 = K2 - 1
         INDXP( K2 ) = J
         COLTYP( J ) = 4
      ELSE
*
*        Check if eigenvalues are close enough to allow deflation.
*
         S = Z( JPREV )
         C = Z( J )
*
*        Find sqrt(a**2+b**2) without overflow or
*        destructive underflow.
*
         TAU = SLAPY2( C, S )
         T = D( J ) - D( JPREV )
         C = C / TAU
         S = -S / TAU
         IF( ABS( T*C*S ) .LE. TOL ) THEN
*
*           Deflation is possible.
*
            Z( J ) = TAU
            Z( JPREV ) = ZERO
            IF ( COLTYP(J) .NE. COLTYP(JPREV) ) 
     $           COLTYP(J) = 3
            COLTYP( JPREV ) = 4
            CALL SROT( N, Q( 1, INDXQ( INDX( JPREV ) ) ), 1, 
     $           Q( 1, INDXQ( INDX( J ) ) ), 1, C, S )
            T = D( JPREV )*C*C + D( J )*S*S
            D( J ) = D( JPREV )*S*S + D( J )*C*C
            D( JPREV ) = T
            K2 = K2 - 1
            I = 1
 200        IF ( K2+I .LE. N ) THEN
               IF ( D( JPREV ) .LT. D( INDXP(K2+I) ) ) THEN
                  INDXP( K2+I-1 ) = INDXP( K2+I )
                  INDXP( K2+I ) = JPREV
                  I = I + 1
                  GO TO 200
               ELSE
                  INDXP( K2+I-1 ) = JPREV
               END IF
            ELSE
               INDXP( K2+I-1 ) = JPREV
            END IF
            JPREV = J
         ELSE
            K = K + 1
            W( K ) = Z( JPREV )
            DLAMDA( K ) = D( JPREV )
            INDXP( K ) = JPREV
            JPREV = J
         END IF
      END IF
      GO TO 60
   80 CONTINUE
*
*     Record the last eigenvalue.
*
      K = K + 1
      W( K ) = Z( JPREV )
      DLAMDA( K ) = D( JPREV )
      INDXP( K ) = JPREV
*
   90 CONTINUE
*
*     Count up the total number of the various types of columns, then
*     form a permutation which positions the four column types into
*     four uniform groups (although one or more of these groups may be
*     empty).
*
      DO 95 J = 1, 4
         CTOT(J) = 0
 95   CONTINUE
      DO 120 J = 1, N
         CT = COLTYP( J )
         CTOT( CT ) = CTOT( CT ) + 1
  120 CONTINUE
*
*     PSM(*) = Position in SubMatrix (of types 1 through 4)
*
      PSM( 1 ) = 1
      PSM( 2 ) = 1 + CTOT( 1 )
      PSM( 3 ) = PSM( 2 ) + CTOT( 2 )
      PSM( 4 ) = PSM( 3 ) + CTOT( 3 )
*
*     Fill out the INDXC array so that the permutation which it induces
*     will place all type-1 columns first, all type-2 columns next,
*     then all type-3's, and finally all type-4's.
*
      DO 130 J = 1, N
         JP = INDXP( J )
         CT = COLTYP( JP )
         INDXC( PSM( CT ) ) = J
         PSM( CT ) = PSM( CT ) + 1
  130 CONTINUE
*
*     Sort the eigenvalues and corresponding eigenvectors into DLAMDA and Q2
*     respectively.  The eigenvalues/vectors which were not deflated go 
*     into the first K slots of DLAMDA and Q2 respectively, while those
*     which were deflated go into the last N - K slots.
*
      DO 100 J = 1, N
         JP = INDXP( J )
         DLAMDA( J ) = D( JP )
         CALL SCOPY( N, Q( 1, INDXQ(INDX(INDXP(INDXC( J ) )))), 1, 
     $        Q2( 1, J ), 1 )
 100  CONTINUE
*
*     The deflated eigenvalues and their corresponding vectors go back into
*     the last N - K slots of D and Q respectively.
*
      CALL SCOPY( N - K, DLAMDA( K+1 ), 1, D( K+1 ), 1 )
      CALL SLACPY( 'A', N, N - K, Q2( 1, K+1 ), LDQ2, 
     $             Q( 1, K+1 ), LDQ )
*
*     Copy CTOT into COLTYP for referencing in SLAED3.
*
      DO 140 J = 1, 4
         COLTYP( J ) = CTOT( J )
  140 CONTINUE
*
      RETURN
*
*     End of SLAED2
*
      END
