      SUBROUTINE  SLAED3( K, KSTART, KSTOP, N, Q, LDQ, 
     $                    D, RHO, DLAMDA, Q2, LDQ2, W, S, LDS, 
     $                    INDXC, CUTPNT, CTOT, INFO )
*
*  -- LAPACK routine (version 2.0) --
*     Univ. of Tennessee, Oak Ridge National Lab, Argonne National Lab,
*     Courant Institute, NAG Ltd., and Rice University
*     January 15, 1994
*
*     .. Scalar Arguments ..
      INTEGER            INFO, K, KSTART, KSTOP, LDQ, LDQ2, LDS, 
     $                   N, CUTPNT
      REAL               RHO
*     ..
*     .. Array Arguments ..
      INTEGER            INDXC( * ), CTOT( * )
      REAL               D( * ), DLAMDA( * ), Q( LDQ, * ), 
     $                   Q2( LDQ2, * ), S( LDS, * ), W( * )
*     ..
*
*  Purpose
*  =======
*
*  SLAED3 finds the roots of the secular equation, as defined by the 
*  values in D, Z, and RHO, between KSTART and KSTOP.  It makes the 
*  appropriate calls to SLAED4 and then updates the eigenvectors by 
*  multiplying the matrix of eigenvectors of the pair of eigensystems
*  being combined by the matrix of eigenvectors of the K x K system 
*  which is solved here.
*
*  This code makes very mild assumptions about floating point
*  arithmetic. It will work on machines with a guard digit in
*  add/subtract, or on those binary machines without guard digits
*  which subtract like the Cray XMP, Cray YMP, Cray C 90, or Cray 2.
*  It could conceivably fail on hexadecimal or decimal machines 
*  without guard digits, but we know of none.
*
*  Arguments
*  =========
*
*  K       (input) INTEGER
*          The number of terms in the rational function to be solved by
*          SLAED4.  K >= 0.
*
*  KSTART  (input) INTEGER
*  KSTOP   (input) INTEGER
*          The updated eigenvalues Lambda(I), KSTART <= I <= KSTOP
*          are to be computed.  1 <= KSTART <= KSTOP <= K.
*
*  N       (input) INTEGER
*          The number of rows and columns in the Q matrix.
*          N >= K (deflation may result in N>K).
*
*  Q       (output) REAL array, dimension (LDQ,N)
*          Initially the first K columns are used as workspace.
*          On output the columns KSTART to KSTOP contain
*          the updated eigenvectors.
*
*  LDQ     (input) INTEGER
*          The leading dimension of the array Q.  LDQ >= max( 1, N ).
*
*  D       (output) REAL array, dimension(N)
*          On output D(I) contains the updated eigenvalues
*          for KSTART <= I <= KSTOP.
*
*  RHO     (input) REAL
*          This scalar contains the value of the parameter in
*          the rank one update equation.  RHO >= 0 required.
*
*  DLAMDA  (input) REAL array, dimension(K)
*          The first K elements of this array contain the old roots
*          of the deflated updating problem.  These are the poles
*          of the secular equation.
*
*  Q2      (input) REAL array, dimension (LDQ2, N)
*          On input the first K columns of this matrix contain the 
*          non-deflated eigenvectors for the split problem.
*
*  LDQ2    (input) INTEGER
*          The leading dimension of the array Q2.  LDQ2 >= max( 1, N ).
*
*  W       (input) REAL array, dimension (K)
*          The first K elements of this array contain the components
*          of the deflation-adjusted updating vector.
*
*  S       (workspace) REAL array, dimension (LDS, K)
*          Will contain the eigenvectors of the repaired matrix which 
*          will be multiplied by the previously accumulated eigenvectors
*          to update the system.
*
*  LDS     (input) INTEGER
*          The leading dimension of S.  LDS >= max( 1, K ).
*
*  INDXC   (input) INTEGER array, dimension ( N )
*          This contains the permutation used to arrange the columns
*          of the deflated Q matrix into three groups:  the first group 
*          contains non-zero entries only at and above CUTPNT, the second
*          contains non-zero entries only below CUTPNT, and the third is
*          dense.  The rows of the eigenvectors found by SLAED4 must be
*          likewise permuted before the matrix multiply can take place.
*
*  CUTPNT  (input) INTEGER
*          Contains the location of the last eigenvalue in the leading
*          sub-matrix.  MIN(1,N) <= CUTPNT <= N.
*
*  CTOT    (input) INTEGER array, dimension ( 4 )
*          CTOT contains a count of the total number of the various types of 
*          columns in Q, as described in INDXC.  The fourth column type
*          is any column which has been deflated.
*
*  INFO    (return) INTEGER
*          = 0:  successful exit.
*          < 0:  if INFO = -i, the i-th argument had an illegal value.
*          > 0:  if INFO = 1, an eigenvalue did not converge
*
*  =====================================================================
*
*     .. Parameters ..
      REAL               ONE, ZERO
      PARAMETER          ( ONE = 1.0E0, ZERO = 0.0E0 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, J, JC, KTEMP, PARTS
      REAL               TEMP
*     ..
*     .. External Functions ..
      REAL               SNRM2, SLAMC3
      EXTERNAL           SNRM2, SLAMC3
*     ..
*     .. External Subroutines ..
      EXTERNAL           SCOPY, SGEMM, SLAED4, SLASET, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, SQRT, SIGN
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
      IF( K .LT. 0 ) THEN
         INFO = -1
      ELSE IF ( KSTART .LT. 1  .OR.  KSTART .GT. MAX(1,K) ) THEN
         INFO = -2
      ELSE IF ( MAX(1,KSTOP) .LT. KSTART  .OR.  
     $          KSTOP .GT. MAX(1,K) ) THEN
         INFO = -3
      ELSE IF ( N .LT. K ) THEN
         INFO = -4
      ELSE IF ( LDQ .LT. MAX( 1, N ) ) THEN
         INFO = -6
      ELSE IF ( LDQ2 .LT. MAX( 1, N ) ) THEN
         INFO = -11
      ELSE IF ( LDS .LT. MAX( 1, K ) ) THEN
         INFO = -14
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'SLAED3', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF ( K .EQ. 0 ) 
     $     RETURN
*
      KTEMP = KSTOP - KSTART + 1
      DO 10 J = KSTART, KSTOP
         CALL SLAED4( K, J, DLAMDA, W, Q( 1, J ), RHO, D( J ), INFO )
*
*        If the zero finder fails, the computation is terminated.
*
         IF( INFO .NE. 0 ) THEN
            RETURN
         END IF
 10   CONTINUE
*
*     Modify values DLAMDA(i) to make sure all DLAMDA(i)-DLAMDA(j) can
*     be computed with high relative accuracy (barring over/underflow).
*     This is a problem on machines without a guard digit in
*     add/subtract (Cray XMP, Cray YMP, Cray C 90 and Cray 2).
*     The following code replaces DLAMDA(I) by 2*DLAMDA(I)-DLAMDA(I),
*     which on any of these machines zeros out the bottommost
*     bit of DLAMDA(I) if it is 1; this makes the subsequent
*     subtractions DLAMDA(I)-DLAMDA(J) unproblematic when cancellation
*     occurs. On binary machines with a guard digit (almost all
*     machines) it does not change DLAMDA(I) at all. On hexadecimal
*     and decimal machines with a guard digit, it slightly
*     changes the bottommost bits of DLAMDA(I). It does not account
*     for hexadecimal or decimal machines without guard digits
*     (we know of none). We use a subroutine call to compute
*     2*DLAMBDA(I) to prevent optimizing compilers from eliminating
*     this code.
*
      DO 15 I = 1, N
         DLAMDA(I) = SLAMC3( DLAMDA(I), DLAMDA(I) ) - DLAMDA(I)
 15   CONTINUE
*
      IF ( K .EQ. 1 .OR. K .EQ. 2 ) THEN
         DO 30  I = 1, K
            DO 20  J = 1, K
               JC = INDXC( J )
               S( J, I ) = Q( JC, I )
 20         CONTINUE
 30      CONTINUE
         GO TO 120
      END IF
*
*     Compute updated W.
*
      CALL SCOPY( K, W, 1, S, 1 )
*
*     Initialize W(I) = Q(I,I)
*
      CALL SCOPY( K, Q, LDQ+1, W, 1 )
      DO 70 J = 1, K
         DO 50 I = 1, J-1
               W( I ) = W( I ) *
     $              ( Q( I, J ) / ( DLAMDA( I )-DLAMDA( J ) ) )
 50      CONTINUE
         DO 60 I = J+1, K
            W( I ) = W( I ) *
     $              ( Q( I, J ) / ( DLAMDA( I )-DLAMDA( J ) ) )
 60      CONTINUE
 70   CONTINUE
      DO  75 I = 1, K
          W( I ) =  SIGN( SQRT( -W( I ) ), S( I, 1 ) )
 75   CONTINUE
*
*     Compute eigenvectors of the modified rank-1 modification.
*
      DO 100 J = 1, K
         DO 80 I = 1, K
	    Q( I, J ) = W( I ) / Q( I, J )
 80      CONTINUE
	 TEMP = SNRM2( K, Q( 1, J ), 1 )
	 DO 90 I = 1, K
            JC = INDXC( I )
	    S( I, J ) = Q( JC, J ) / TEMP
 90      CONTINUE
 100  CONTINUE
*
*     Compute the updated eigenvectors.
*
 120  CONTINUE
*
      PARTS = 0
      IF ( CTOT(1) .GT. 0 ) THEN
         PARTS = PARTS + 1
         CALL SGEMM( 'N', 'N', CUTPNT, KTEMP, CTOT(1), ONE, 
     $        Q2( 1, 1 ), LDQ2, 
     $        S( 1, KSTART ), LDS, 
     $        ZERO, Q( 1, KSTART ), LDQ )
      END IF
      IF ( CTOT(2) .GT. 0 ) THEN
         PARTS = PARTS + 2
         CALL SGEMM( 'N', 'N', N - CUTPNT, KTEMP, CTOT(2), ONE, 
     $        Q2( 1 + CUTPNT, 1 + CTOT(1) ), LDQ2, 
     $        S( 1 + CTOT(1), KSTART ), LDS, 
     $        ZERO, Q( 1 + CUTPNT, KSTART ), LDQ )
      END IF
      IF ( PARTS .EQ. 1 ) 
     $     CALL SLASET( 'A', N - CUTPNT, KTEMP, 
     $                 ZERO, ZERO, Q( 1+CUTPNT, KSTART ), LDQ )
      IF ( PARTS .EQ. 2 ) 
     $     CALL SLASET( 'A', CUTPNT, KTEMP,
     $                 ZERO, ZERO, Q( 1, KSTART ), LDQ )
      IF ( CTOT(3) .GT. 0 ) THEN
         IF ( PARTS .GT. 0 ) THEN
            CALL SGEMM( 'N', 'N', N, KTEMP, CTOT(3), ONE, 
     $           Q2( 1, 1 + CTOT(1) + CTOT(2) ), LDQ2, 
     $           S( 1 + CTOT(1) + CTOT(2), KSTART ), LDS, 
     $           ONE, Q( 1, KSTART ), LDQ )
         ELSE
            CALL SGEMM( 'N', 'N', N, KTEMP, CTOT(3), ONE, 
     $           Q2( 1, 1 + CTOT(1) + CTOT(2) ), LDQ2, 
     $           S( 1 + CTOT(1) + CTOT(2), KSTART ), LDS, 
     $           ZERO, Q( 1, KSTART ), LDQ )
         END IF
      END IF
*
      RETURN
*
*     End of SLAED3
*
      END
