      SUBROUTINE  SLAED5( I, D, Z, DELTA, RHO, DLAM )
*
*  -- LAPACK routine (version 2.0) --
*     Univ. of Tennessee, Oak Ridge National Lab, Argonne National Lab,
*     Courant Institute, NAG Ltd., and Rice University
*     January 15, 1994
*
*     .. Scalar Arguments ..
      INTEGER            I
      REAL               DLAM, RHO
*     ..
*     .. Array Arguments ..
      REAL               D( 2 ), DELTA( 2 ), Z( 2 )
*     ..
*
*  Purpose
*  =======
*
*  This subroutine computes the I-th eigenvalue of a symmetric rank-one
*  modification of a 2-by-2 diagonal matrix
*
*             diag( D )  +  RHO *  Z * transpose(Z) .
*
*  The diagonal entries in the array D are assumed to satisfy
*
*             D(i) < D(j)  for  i < j .
*
*  We also assume RHO > 0 and that the Euclidean norm of the vector
*  Z is one.
*
*  Arguments
*  =========
*
*  I      (input) INTEGER
*         The index of the eigenvalue to be computed.  I = 1 or I = 2.
*
*  D      (input) REAL array, dimension ( 2 )
*         Contains the original eigenvalues.  We assume D(1) < D(2).
*
*  Z      (input) REAL array, dimension ( 2 )
*         Contains the components of the updating vector.
*
*  DELTA  (output) REAL array, dimension ( 2 )
*         Contains (D(j) - lambda_I) in its  j-th component.
*         The vector DELTA contains the information necessary
*         to construct the eigenvectors.
*
*  RHO    (input) REAL
*         This is the scalar in the the symmetric updating
*         formula.
*
*  DLAM   (output) REAL
*         This scalar will contain the computed lambda_I, the
*         I-th updated eigenvalue, on return.
*
*  =====================================================================
*
*     .. Parameters ..
      REAL               ZERO, ONE, TWO, FOUR
      PARAMETER          ( ZERO = 0.0E0, ONE = 1.0E0, TWO = 2.0E0,
     $                   FOUR = 4.0E0 )
*     ..
*     .. Local Scalars ..
      REAL               B, C, DEL, W, TEMP, TAU
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, SQRT
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      DEL =  D(2)-D(1) 
      IF (I.EQ.1) THEN
         W=ONE+TWO*RHO*(Z(2)*Z(2)-Z(1)*Z(1) )/DEL
         IF( W.GT.ZERO) THEN
            B = DEL+RHO*(Z(1)*Z(1)+Z(2)*Z(2))
            C = RHO*Z(1)*Z(1)*DEL
*
*           B > ZERO, always
*
            TAU = TWO*C/(B+SQRT(ABS(B*B-FOUR*C)))
            DLAM = D(1)+TAU
            DELTA(1)=-Z(1)/TAU
            DELTA(2)=Z(2)/(DEL-TAU)
         ELSE
            B = -DEL+RHO*(Z(1)*Z(1)+Z(2)*Z(2))
            C = RHO*Z(2)*Z(2)*DEL
            IF( B.GT.ZERO) THEN
               TAU = -TWO*C/(B+SQRT(B*B+FOUR*C))
            ELSE
               TAU = (B-SQRT(B*B+FOUR*C))/TWO
            END IF
            DLAM = D(2)+TAU
            DELTA(1) = -Z(1)/(DEL+TAU)
            DELTA(2) = -Z(2)/TAU
         END IF
         TEMP = SQRT(DELTA(1)*DELTA(1)+DELTA(2)*DELTA(2))
         DELTA(1)=DELTA(1)/TEMP
         DELTA(2)=DELTA(2)/TEMP
      ELSE
*
*     Now I=2
*
         B = -DEL+RHO*(Z(1)*Z(1)+Z(2)*Z(2))
         C = RHO*Z(2)*Z(2)*DEL
         IF (B.GT.ZERO) THEN
            TAU = (B+SQRT(B*B+FOUR*C))/TWO
         ELSE
            TAU = TWO*C/(-B+SQRT(B*B+FOUR*C))
         END IF
         DLAM = D(2)+TAU
         DELTA(1) = -Z(1)/(DEL+TAU)
         DELTA(2) = -Z(2)/TAU
         TEMP = SQRT(DELTA(1)*DELTA(1)+DELTA(2)*DELTA(2))
         DELTA(1)=DELTA(1)/TEMP
         DELTA(2)=DELTA(2)/TEMP
      END IF
*
*     End OF SLAED5
*
      RETURN       
      END
