      SUBROUTINE SLAED6( KNITER, ORGATI, RHO, D, Z, FINIT, TAU, INFO )
*
*  -- LAPACK routine (version 2.0) --
*     Univ. of Tennessee, Oak Ridge National Lab, Argonne National Lab,
*     Courant Institute, NAG Ltd.,.AND.Rice University
*     January 15, 1994
*
*     .. Scalar Arguments ..
      LOGICAL            ORGATI
      INTEGER            KNITER, INFO
      REAL               TAU, FINIT, RHO
*     ..
*     .. Array Arguments ..
      REAL               D(3), Z(3)
*     ..
*
*  Purpose
*  =======
*
*  SLAED6 computes the positive or negative root (closest to the origin)
*  of 
*                   z(1)        z(2)        z(3)
*  f(x) =   rho + --------- + ---------- + ---------       (*)
*                  d(1)-x      d(2)-x      d(3)-x
*
*  It is assumed that
*
*        if ORGATI = .true. the root is between d(2) and d(3);    
*        otherwise it is between d(1) and d(2)
*   
*
*  This routine will be called by SLAED4 when necessary. In most cases,
*  the root sought is the smallest in magnitude, though it might not be
*  in some extremely rare situations.
*
*  Arguments
*  =========
*
*  KNITER       (input) INTEGER scalar
*               Refer to SLAED4 for its significance.
*
*  ORGATI       (input) LOGICAL scalar (cf. SLAED4)
*               if ORGATI is true, the needed root is between d(2) and d(3);
*               otherwise it is between d(1) and d(2).
*
*  RHO          (input) REAL scalar
*               Refer to the equation (*) above.
*
*  D            (input) REAL array, dimension(3)
*               d is not changed upon exit. D satisfies
*
*                      d(1) < d(2) < d(3)
*
*  Z            (input) REAL array, dimension(3)
*               z is not changed upon exit. Each entries of
*               z is positive.
*
*  FINIT        (input) REAL scalar
*               The value of f at 0. It is more accurate than the one
*               evaluated inside this routine (if someone wants to do
*               so).
*
*  TAU          (output) REAL
*               The root asked. 
*
*
*  INFO         (output) INTEGER
*               = 0: successful output
*               = 1: failure to converge
*
*  =====================================================================
*
*
*     .. Parameters ..
      REAL                ZERO, ONE, TWO, THREE, FOUR, EIGHT
      PARAMETER          ( ZERO = 0.0E0, ONE = 1.0E0, TWO = 2.0E0,
     $                     THREE = 3.0E0, FOUR = 4.0E0, 
     $                     EIGHT = 8.0E0 )
*     ..
*     .. External Functions ..
      REAL               SLAMCH
      EXTERNAL           SLAMCH
*     ..
*     .. Local Scalars ..
      INTEGER            NITER, MAXIT, I
      REAL               F, FC, DF, DDF, A, B, C,
     $                   TEMP, EPS, ERRETM, TEMP1, TEMP2, TEMP3,
     $                   ETA, PRETAU
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, SQRT
*     ..
*     .. Executable Statements ..
*     
      MAXIT = 20
      INFO = 0
      EPS = SLAMCH( 'Epsilon' )         
*
      NITER = 1
      TAU = ZERO
      IF (KNITER .EQ. 2) THEN
         IF (ORGATI) THEN
            TEMP = (D(3)-D(2))/TWO
            C = RHO + Z(1)/( (D(1)-D(2))-TEMP )
            A = C*(D(2)+D(3))+Z(2)+Z(3)
            B = C*D(2)*D(3)+Z(2)*D(3)+Z(3)*D(2)
         ELSE
            TEMP = (D(1)-D(2))/TWO
            C = RHO + Z(3)/( (D(3)-D(2)) -TEMP )
            A = C*(D(1)+D(2))+Z(1)+Z(2)
            B = C*D(1)*D(2)+Z(1)*D(2)+Z(2)*D(1)
         END IF
         IF ( C .EQ. ZERO ) THEN
            TAU = B/A
         ELSE IF ( A .LE. ZERO ) THEN
            TAU = (A - SQRT(ABS(A*A - FOUR*B*C)))/(TWO*C)
         ELSE
            TAU = TWO*B/(A+SQRT(ABS(A*A - FOUR*B*C)))
         END IF        
         TEMP = RHO + Z(1)/(D(1)-TAU) + Z(2)/(D(2)-TAU)
     $              + Z(3)/(D(3)-TAU)
         IF (ABS(FINIT) .LE. ABS(TEMP)) TAU = ZERO
      END IF
*
      FC = ZERO
      DF = ZERO
      DDF = ZERO
      DO 30 I = 1, 3
	 TEMP  = ONE/(D(I)-TAU)
	 TEMP1 = Z(I) * TEMP
	 TEMP2 = TEMP1 * TEMP
	 TEMP3 = TEMP2 * TEMP
	 FC = FC + TEMP1/D(I)
	 DF = DF + TEMP2
	 DDF = DDF + TEMP3
 30   CONTINUE
      F = FINIT + TAU*FC
*
      IF ( ABS(F) .LE. ZERO)  RETURN
*
*        Iteration begins
*
*     It is not hard to see that
*
*           1) Iterations will go up monotonically
*              if FINIT < 0;
*
*           2) Iterations will go down monotonically
*              if FINIT > 0.
*
 40      CONTINUE
         IF ( NITER .GT. MAXIT ) THEN
	    INFO = 1
            RETURN
         END IF
         NITER = NITER + 1
         IF (ORGATI) THEN
            TEMP1 = D(2) - TAU
            TEMP2 = D(3) - TAU
         ELSE
            TEMP1 = D(1) - TAU
            TEMP2 = D(2) - TAU
         END IF
         A = (TEMP1+TEMP2)*F - TEMP1*TEMP2*DF
         B = TEMP1*TEMP2*F
         C = F - (TEMP1+TEMP2)*DF+TEMP1*TEMP2*DDF
         IF (C.EQ.ZERO) THEN
            ETA=B/A
         ELSE IF(A.LE.ZERO) THEN
            ETA=(A-SQRT(ABS(A*A-FOUR*B*C)))/(TWO*C)
         ELSE
            ETA=TWO*B/(A+SQRT(ABS(A*A-FOUR*B*C)))
         END IF
         IF (F*ETA .GE. ZERO ) THEN
            ETA = - F/DF
         END IF
*
         TEMP=ETA+TAU
         IF (ORGATI) THEN
            IF (ETA .GT. ZERO .AND. TEMP .GE. D(3))
     $             ETA = (D(3)-TAU)/TWO
            IF (ETA .LT. ZERO .AND. TEMP .LE. D(2))
     $             ETA = (D(2)-TAU)/TWO
         ELSE
            IF (ETA .GT. ZERO .AND. TEMP .GE. D(2))
     $             ETA = (D(2)-TAU)/TWO
            IF (ETA .LT. ZERO .AND. TEMP .LE. D(1))
     $             ETA = (D(1)-TAU)/TWO
         END IF
         PRETAU = TAU                 
         TAU = TAU + ETA
*
         FC = ZERO
         ERRETM = ABS( RHO )
         DF = ZERO
         DDF = ZERO
         DO 50 I = 1, 3
            TEMP  = ONE/(D(I)-TAU)
            TEMP1 = Z(I) * TEMP
            TEMP2 = TEMP1 * TEMP
            TEMP3 = TEMP2 * TEMP
            FC = FC + TEMP1/( D(I)-PRETAU )
            ERRETM = ERRETM + ABS(TEMP1)
            DF = DF + TEMP2
            DDF = DDF + TEMP3
 50      CONTINUE
	 F = F + ETA*FC
         ERRETM = EIGHT*ERRETM + ABS(TAU)*DF 
         IF ( ABS(F) .LE. EPS*ERRETM ) THEN
	    RETURN
	 END IF
         GO TO 40
*
*     End of SLAED6
*
      END
