      SUBROUTINE  SLAMRG( N1, N2, A, STRD1, STRD2, INDEX )
*
*  -- LAPACK routine (version 2.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     January 15, 1994
*
*     .. Scalar Arguments ..
      INTEGER           N1, N2, STRD1, STRD2
*     ..
*     .. Array Arguments ..
      INTEGER           INDEX( * )
      REAL              A( * )
*     ..
*
*  Purpose
*  =======
*
*  SLAMRG will create a permutation list which will merge the entries of A 
*  (which is composed of two independently sorted sets) into a single 
*  set which is sorted in ascending order.
*
*  Arguments
*  =========
*
*  N1     (input) INTEGER
*  N2     (input) INTEGER
*         These arguements contain the respective lengths of the two 
*         sorted lists to be merged.
*
*  A      (input) REAL array, dimension ( N1 + N2 )
*         The first N1 entries of A contain a list of numbers which
*         are sorted in either ascending or descending order.  Likewise
*         for the final N2 entries.
*
*  STRD1  (input) INTEGER     
*  STRD2  (input) INTEGER     
*         These are the strides to be taken through the array A.  
*         Allowable strides are 1 and -1.  They indicate whether a subset of
*         A is sorted in ascending (STRDx = 1) or descending (STRDx = -1) 
*         order.  
*
*  INDEX  (output) INTEGER array, dimension ( N1 + N2 )
*         On exit this array will contain a permutation such that
*         if B( I ) = A( INDEX( I ) ) for I=1,N1+N2, then B will be sorted
*         in ascending order.
*
*  =====================================================================
*
*     .. Local Scalars ..
      INTEGER         I, IND1, IND2
*     ..
*     .. Executable Statements ..
*
      IF ( STRD1 .GT. 0 ) THEN
         IND1 = 1
      ELSE
         IND1 = N1
      END IF
      IF ( STRD2 .GT. 0 ) THEN
         IND2 = 1 + N1
      ELSE
         IND2 = N1 + N2
      END IF
      I = 1
*     while ( (N1 > 0) & (N2 > 0) ) 
 10   IF ( N1 .GT. 0  .AND.  N2 .GT. 0 ) THEN
         if ( A( IND1 ) .LE. A( IND2 ) ) THEN
            INDEX( I ) = IND1
            I = I + 1
            IND1 = IND1 + STRD1
            N1 = N1 - 1
         ELSE
            INDEX( I ) = IND2
            I = I + 1
            IND2 = IND2 + STRD2
            N2 = N2 - 1
         END IF
         GO TO 10
      END IF
*     end while
      IF ( N1 .EQ. 0 ) THEN
         DO 100  N1 = 1, N2
            INDEX( I ) = IND2
            I = I + 1
            IND2 = IND2 + STRD2
 100     CONTINUE
      ELSE
*     N2 .EQ 0
         DO 120  N2 = 1, N1
            INDEX( I ) = IND1
            I = I + 1
            IND1 = IND1 + STRD1
 120     CONTINUE
      END IF
*
      RETURN
*
*     End of SLAMRG
*
      END
