      REAL FUNCTION SLARAN( ISEED )
*
*  -- LAPACK auxiliary routine (preliminary version) --
*     Univ. of Tennessee, Oak Ridge National Lab, Argonne National Lab,
*     Courant Institute, NAG Ltd., and Rice University
*     March 26, 1990
*
*     .. Array Arguments ..
      INTEGER            ISEED( 4 )
*     ..
*
*  Purpose
*  =======
*
*  ISEED generates and returns random numbers uniformly distributed
*  between 0. and 1.  A linear congruential sequence is used.
*
*  This code is machine independent provided 12 bit integers can be
*  added and multiplied to produce 24 bit answers.  Note that ISEED(4)
*  must be odd.
*
*  Arguments
*  =========
*
*  ISEED   (input/output) INTEGER array, dimension (4)
*          On entry, the seed of the random number generator. The array
*          elements should be between 0 and 4095; if not they will be
*          reduced mod 4096.  Also, ISEED(4) must be odd.  The random
*          number generator uses a linear congruential sequence limited
*          to small integers, and so should produce machine independent
*          random numbers.
*          On exit, ISEED is changed so that the next call will generate
*          a different number.
*
*
*     .. Parameters ..
      INTEGER            M1
      PARAMETER          ( M1 = 502 )
      INTEGER            M2
      PARAMETER          ( M2 = 1521 )
      INTEGER            M3
      PARAMETER          ( M3 = 4071 )
      INTEGER            M4
      PARAMETER          ( M4 = 2107 )
      REAL   ONE
      PARAMETER          ( ONE = 1.0E0 )
      REAL   T12
      PARAMETER          ( T12 = 4096.0E0 )
*     ..
*     .. Local Scalars ..
      INTEGER            I1, I2, I3, I4
      REAL               R
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          REAL, MOD
*     ..
*     .. Executable Statements ..
*
*     The following is just multiplication of two 48-bit
*     fixed-point numbers, each of which is represented by 4 12-bit
*     pieces.  The constant, "M", is represented by M1 throught M4,
*     M1 being the high-order part, and the variable ISEED is
*     represented by ISEED(1) through ISEED(4), ISEED(1) being the
*     high-order part.  The binary point can be thought of as
*     lying between M1 and M2, and between ISEED(1) and ISEED(2).
*
*     The code is thus ISEED = MOD( M * ISEED , 4096 )
*
      I1 = ISEED( 1 )*M4 + ISEED( 2 )*M3 + ISEED( 3 )*M2 + ISEED( 4 )*M1
      I2 = ISEED( 2 )*M4 + ISEED( 3 )*M3 + ISEED( 4 )*M2
      I3 = ISEED( 3 )*M4 + ISEED( 4 )*M3
      I4 = ISEED( 4 )*M4
      ISEED( 4 ) = MOD( I4, 4096 )
      I3 = I3 + I4 / 4096
      ISEED( 3 ) = MOD( I3, 4096 )
      I2 = I2 + I3 / 4096
      ISEED( 2 ) = MOD( I2, 4096 )
      ISEED( 1 ) = MOD( I1+I2/4096, 4096 )
*
*     Compute SLARAN = ISEED / 4096.0
*
      R = ONE / T12
      SLARAN = R*( REAL( ISEED( 1 ) )+R*
     $         ( REAL( ISEED( 2 ) )+R*( REAL( ISEED( 3 ) )+R*
     $         ( REAL( ISEED( 4 ) ) ) ) ) )
      RETURN
*
*     End of DLARAN
*
      END
