*
*     Order eigenvalues and eigenvectors.
*
      SUBROUTINE SLASRT( ID, N, D, INFO )
*
*  -- LAPACK routine (version 2.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     February 29, 1992
*
*     .. Scalar Arguments ..
      CHARACTER          ID
      INTEGER            N, INFO
*     ..
*     .. Array Arguments ..
      REAL               D( * )
*     ..
*
*  Purpose
*  =======
*
*  Sort the numbers in D in increasing order (if ID = 'I') or
*  in decreasing order (if ID = 'D' ).
*
*  Use Quick Sort, reverting to Insertion sort on arrays of
*  size <= 20. Dimension of STACK limits N to about 2**32.
*
*  Arguments
*  =========
*
*  ID      (input) CHARACTER*1
*          On entry, ID specifies how to sort D.
*             ID = 'I' or 'i' means sort in increasing order
*             ID = 'D' or 'd' means sort in decreasing order
*
*  N       (input) INTEGER
*          On entry, N specifies the length of array D
*
*  D       (input/output) REAL array, dimension ( N )
*          On entry, N contains real numbers.
*          On exit, these have been sorted into increasing order 
*          (D(1) <= ... <= D(N) ) or into decreasing order
*          (D(1) >= ... >= D(N) ), depending on ID. 
*
*  INFO    (output) INTEGER
*          On exit, a value of 0 indicates a successful exit.
*          If INFO < 0, argument number -INFO is illegal.
*
*  =====================================================================
*
*     ..
*     .. Parameters ..
      INTEGER            SELECT
      PARAMETER          ( SELECT = 20 )
*     ..
*     .. Local Scalars ..
      INTEGER            DIR, ENDD, I, J, LOC, START, STKPNT 
      REAL               DMNMX, TMP, D1, D2, D3
*     ..
*     .. Local Arrays ..
      INTEGER            STACK( 2, 32 )
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     ..
*     .. Executable Statements ..
*
*     Test the input paramters.
*
      INFO = 0
      DIR = -1
      IF ( LSAME( ID, 'D') ) THEN
         DIR = 0
      ELSEIF ( LSAME ( ID, 'I' ) ) THEN
         DIR = 1
      ENDIF
      IF ( DIR .EQ. -1 ) THEN
         INFO = -1
      ELSEIF ( N .LT. 0 ) THEN
         INFO = -2
      ENDIF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'SLASRT', -INFO )
         RETURN
      END IF
*
      IF (N .LE. 1) RETURN
      STKPNT = 1
      STACK( 1, 1 ) = 1
      STACK( 2, 1 ) = N
10    CONTINUE
      START = STACK( 1, STKPNT )
      ENDD = STACK( 2, STKPNT )
      STKPNT = STKPNT - 1
      IF ( ENDD - START .LE. SELECT .AND. ENDD - START .GT. 0 ) THEN
*
*        Do Insertion sort on D( START:ENDD )
*
         IF ( DIR .EQ. 0 ) THEN
*
*           Sort into decreasing order
*
            DO 20 I = START+1, ENDD
               DO 30 J = I, START+1, -1
                  IF ( D( J ) .GT. D( J-1 ) ) THEN
                     DMNMX = D( J )
                     D( J ) = D( J-1 )
                     D( J-1 ) = DMNMX
                  ELSE
                     GOTO 20
                  ENDIF
30             CONTINUE
20          CONTINUE
*
         ELSE
*
*           Sort into increasing order
*
            DO 40 I = START+1, ENDD
               DO 50 J = I, START+1, -1
                  IF ( D( J ) .LT. D( J-1 ) ) THEN
                     DMNMX = D( J )
                     D( J ) = D( J-1 )
                     D( J-1 ) = DMNMX
                  ELSE
                     GOTO 40
                  ENDIF
50             CONTINUE
40          CONTINUE
*
         ENDIF
*
      ELSEIF (ENDD - START .GT. SELECT) THEN
*
*        Partition D( START:ENDD ) and stack parts, largest one first
*
*        Choose partition element as median of 3
*
         D1 = D ( START )
         D2 = D ( ENDD )
         I = (START+ENDD)/2
         D3 = D ( I )
         IF ( D1 .LT. D2 ) THEN
            IF ( D3 .LT. D1 ) THEN
               DMNMX = D1
            ELSE IF ( D3 .LT. D2 ) THEN
               DMNMX = D3
            ELSE
               DMNMX = D2
            ENDIF
         ELSE
            IF ( D3 .LT. D2 ) THEN
               DMNMX = D2
            ELSE IF ( D3 .LT. D1 ) THEN
               DMNMX = D3
            ELSE
               DMNMX = D1
            ENDIF
         ENDIF
*
         IF ( DIR .EQ. 0 ) THEN
*
*           Sort into decreasing order
*
            I = START - 1
            J = ENDD + 1
60          CONTINUE
70          CONTINUE
            J = J - 1
            IF ( D(J) .LT. DMNMX ) GOTO 70
80          CONTINUE
            I = I + 1
            IF ( D(I) .GT. DMNMX ) GOTO 80
            IF ( I .LT. J ) THEN
               TMP = D( I )
               D( I ) = D( J )
               D( J ) = TMP
               GOTO 60
            ENDIF
            IF ( J - START .GT. ENDD - J - 1 ) THEN
               STKPNT = STKPNT + 1
               STACK( 1, STKPNT ) = START
               STACK( 2, STKPNT ) = J
               STKPNT = STKPNT + 1
               STACK( 1, STKPNT ) = J+1
               STACK( 2, STKPNT ) = ENDD
            ELSE
               STKPNT = STKPNT + 1
               STACK( 1, STKPNT ) = J+1
               STACK( 2, STKPNT ) = ENDD
               STKPNT = STKPNT + 1
               STACK( 1, STKPNT ) = START
               STACK( 2, STKPNT ) = J
            ENDIF
         ELSE
*
*           Sort into increasing order
*
            I = START - 1
            J = ENDD + 1
90          CONTINUE
100         CONTINUE
            J = J - 1
            IF ( D(J) .GT. DMNMX ) GOTO 100
110         CONTINUE
            I = I + 1
            IF ( D(I) .LT. DMNMX ) GOTO 110
            IF ( I .LT. J ) THEN
               TMP = D( I )
               D( I ) = D( J )
               D( J ) = TMP
               GOTO 90
            ENDIF
            IF ( J - START .GT. ENDD - J - 1 ) THEN
               STKPNT = STKPNT + 1
               STACK( 1, STKPNT ) = START
               STACK( 2, STKPNT ) = J
               STKPNT = STKPNT + 1
               STACK( 1, STKPNT ) = J+1
               STACK( 2, STKPNT ) = ENDD
            ELSE
               STKPNT = STKPNT + 1
               STACK( 1, STKPNT ) = J+1
               STACK( 2, STKPNT ) = ENDD
               STKPNT = STKPNT + 1
               STACK( 1, STKPNT ) = START
               STACK( 2, STKPNT ) = J
            ENDIF
         ENDIF
      ENDIF
      IF ( STKPNT .GT. 0 ) GOTO 10
      RETURN
*
*     End of SLASRT
*
      END
