      SUBROUTINE  SSTEDC( COMPQ, N, D, E, Q, LDQ, IWORK, WORK, INFO )
*
*  -- LAPACK routine (version 2.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     January 15, 1994
*
*     .. Scalar Arguments ..
      CHARACTER          COMPQ
      INTEGER            N, LDQ, INFO
*     ..
*     .. Array Arguments ..
      INTEGER            IWORK( * )
      REAL               D( * ), E( * ), Q( LDQ, * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  SSTEDC computes all eigenvalues and, optionally, eigenvectors of a
*  symmetric tridiagonal matrix using the divide and conquer method.
*  The eigenvectors of a full or band symmetric matrix can also be found
*  if SSYTRD or SSPTRD or SSBTRD has been used to reduce this matrix to
*  tridiagonal form.
*
*  This code makes very mild assumptions about floating point
*  arithmetic. It will work on machines with a guard digit in
*  add/subtract, or on those binary machines without guard digits
*  which subtract like the Cray XMP, Cray YMP, Cray C 90, or Cray 2.
*  It could conceivably fail on hexadecimal or decimal machines 
*  without guard digits, but we know of none.  See SLAED3 for details.
*
*  The code currently call SSTERF if eigenvalues only are desired, since
*  this is faster, but it can be easily modified to use divide and conquer 
*  (see the comments below).
*
*  Arguments
*  =========
*
*  COMPQ   (input) CHARACTER*1
*          Specifies whether eigenvectors are to be computed
*          as follows
*
*             COMPQ = 'N' or 'n'     Compute eigenvalues only.
*
*             COMPQ = 'I' or 'i'     Compute eigenvectors of
*                                    tridiagonal matrix also.
*
*             COMPQ = 'V' or 'v'     Compute eigenvectors of original
*                                    dense symmetric matrix also.
*                                    On input, Q contains the orthogonal
*                                    matrix used to reduce the original
*                                    matrix to tridiagonal form.
*
*  N       (input) INTEGER
*          The dimension of the symmetric tridiagonal matrix.  N >= 0.
*
*  D       (input/output) REAL array, dimension(N)
*          On entry D contains the main diagonal of the tridiagonal 
*          matrix.  
*          On exit D, if INFO = 0, contains its eigenvalues.
*
*  E       (input) REAL array, dimension(N-1)
*          Contains the subdiagonal entries of the tridiagonal matrix.
*          On exit, E has been destroyed.
*
*  Q       (input/output) REAL array, dimension(LDQ,N)
*          If  COMPQ = 'V' or 'v', then:
*           On entry, Q contains the orthogonal matrix used in the
*           reduction to tridiagonal form.
*          If  COMPQ = 'V' or 'v' or 'I' or 'i', then:
*           On exit, if INFO = 0, Q contains the orthonormal 
*           eigenvectors of the symmetric tridiagonal (or full) matrix.
*          If  COMPQ = 'N' or 'n', then Q is not referenced.
*
*  LDQ     (input) INTEGER
*          The leading dimension of the array Q.  If eigenvectors are
*          desired, then  LDQ >= max( 1, N ).  In any case,  LDQ >= 1.
*
*  IWORK   (workspace) INTEGER array
*          If COMPQ = 'N' then no integer workspace is required.
*          If COMPQ = 'V' then the dimension of IWORK must be at least
*                         ( 6 + 6*N + 5 * N * lg N )
*                         ( lg( N ) = ceiling( log-base-2 ( N ) ) )
*          If COMPQ = 'I' then the dimension of IWORK must be at least
*                         ( 2 + 5 * N ).
*
*  WORK    (workspace) REAL array, 
*          If COMPQ = 'N' then no workspace is required.
*          If COMPQ = 'V' then the dimension of WORK must be at least
*                         ( 1 + 3 * N + 2 * N * lg N + 3 * N**2 ).
*          If COMPQ = 'I' then the dimension of WORK must be at least
*                         ( 1 + 3 * N + 2 * N * lg N + 2 * N**2 ).
*
*  INFO    (output) INTEGER
*          = 0:  successful exit.
*          < 0:  if INFO = -i, the i-th argument had an illegal value.
*          > 0:  The algorithm failed to compute an eigenvalue while
*                working on the submatrix lying in rows and columns
*                INFO/(N+1) through MOD(INFO,N+1).
*
*  =====================================================================
*
*     .. Parameters ..
      INTEGER            SMLSIZ 
      PARAMETER          ( SMLSIZ = 25 )
      REAL               ZERO, ONE
      PARAMETER          ( ZERO = 0.0E0, ONE = 1.0E0 )
*     ..
*     .. Local Scalars ..
      INTEGER            START, STRTRW, END, M, NM1, ICOMPQ, 
     $                   STOREQ, 
     $                   I, J, K, II
      REAL               EPS, TINY, ORGNRM, 
     $                   P
*     ..
*     .. External functions ..
      LOGICAL            LSAME
      REAL               SLAMCH, SLANST
      EXTERNAL           LSAME, SLAMCH, SLANST, SSWAP, SLASRT
*     ..
*     .. External Subroutines ..
      EXTERNAL           SLAED0, SLASCL, SLASET, SSTEQR, SSTERF, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
      IF( LSAME( COMPQ, 'N' ) ) THEN
         ICOMPQ = 0
      ELSE IF( LSAME( COMPQ, 'V' ) ) THEN
         ICOMPQ = 1
      ELSE IF( LSAME( COMPQ, 'I' ) ) THEN
         ICOMPQ = 2
      ELSE
         ICOMPQ = -1
      END IF
      IF( ICOMPQ.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( ( LDQ.LT.1 ) .OR. ( ICOMPQ.GT.0 .AND. LDQ.LT.MAX( 1,
     $         N ) ) ) THEN
         INFO = -6
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'SSTEDC', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF ( N .EQ. 0 ) 
     $     RETURN
      IF ( N .EQ. 1 ) THEN
         IF ( ICOMPQ .NE. 0 ) 
     $      Q(1,1) = ONE
         RETURN
      END IF
*
*     If the following conditional clause is removed, then the routine
*     will use the Divide and Conquer routine to compute only the
*     eigenvalues, which requires (3N + 3N**2) real workspace and 
*     (2 + 5N + 2N lg(N)) integer workspace.  
*     Since on many architectures SSTERF is much faster than any other 
*     algorithm for finding eigenvalues only, it is used here
*     as the default.
*
*     If COMPQ = 'N', use SSTERF to compute the eigenvalues.
*
      IF ( ICOMPQ .EQ. 0 ) THEN
         CALL SSTERF( N, D, E, INFO )
         RETURN
      END IF
*
*     If N is smaller than the minimum divide size (SMLSIZ+1), then solve
*     the problem with another solver.
*
      IF ( N .LE. SMLSIZ ) THEN
         IF ( ICOMPQ .EQ. 0 ) THEN
            CALL SSTERF( N, D, E, INFO )
            RETURN
         ELSE IF ( ICOMPQ .EQ. 2 ) THEN
            CALL SSTEQR( 'I', N, D, E, Q, LDQ, WORK, INFO )
            RETURN
         ELSE
            CALL SSTEQR( 'V', N, D, E, Q, LDQ, WORK, INFO )
            RETURN
         END IF
      END IF
*
*     If COMPQ = 'V', the Q matrix must be stored elsewhere for
*     later use.
*
      IF ( ICOMPQ .EQ. 1 ) THEN
         STOREQ = 1 + N * N
      ELSE
         STOREQ = 1
      END IF
*
      IF ( ICOMPQ .EQ. 2 ) THEN
         CALL SLASET( 'A', N, N, ZERO, ONE, Q, LDQ )
      END IF
*     
*     Scale.
*
      NM1 = N - 1
      ORGNRM = SLANST( 'M', N, D, E )
      IF ( ORGNRM .EQ. ZERO )
     $     RETURN
*
      EPS = SLAMCH( 'Epsilon' )
*
      START = 1
*
*     while ( START <= N )
*
 100  IF ( START .LE. N ) THEN
*
*     Let END be the position of the next subdiagonal entry such that
*     E( END ) <= TINY or END = N if no such subdiagonal exists.  The
*     matrix identified by the entries between START and END constitutes
*     an independent sub-problem.
*
         END = START
 110     IF ( END .LT. N ) THEN
            TINY = EPS * SQRT( ABS( D( END ) ) ) * 
     $           SQRT( ABS( D( END+1 ) ) )
            IF ( ABS( E( END ) ) .GT. TINY ) THEN
               END = END + 1
               GO TO 110
            END IF
         END IF
*
*        (Sub) Problem determined.  Compute its size and solve it.
*
         M = END - START + 1    
         IF ( M .EQ. 1 ) THEN
            START = END + 1
            GO TO 100
         END IF
         IF ( M .GT. SMLSIZ ) THEN
            INFO = SMLSIZ
*
*           Scale.
*
            ORGNRM = SLANST( 'M', M, D( START ), E( START ) )
            CALL SLASCL( 'G', 0, 0, ORGNRM, ONE, M, 1, D( START ), 
     $                   M, INFO )
            CALL SLASCL( 'G', 0, 0, ORGNRM, ONE, M-1, 1, E( START ), 
     $                   M-1, INFO )
*
            IF ( ICOMPQ .EQ. 1 ) THEN
               STRTRW = 1
            ELSE
               STRTRW = START
            END IF
            CALL SLAED0( ICOMPQ, N, M, D( START ), E( START ), 
     $           Q( STRTRW, START ), LDQ, IWORK, WORK(STOREQ), 
     $           WORK( 1 ), N, INFO )
            IF ( INFO .GT. 0 ) THEN
               INFO = (INFO/(M+1)+START-1)*(N+1)
     $              + MOD(INFO,(M+1))+START-1
               RETURN
            END IF
*
*           Scale back.
*
            CALL SLASCL( 'G', 0, 0, ONE, ORGNRM, M, 1, D( START ), 
     $                   M, INFO )
*
         ELSE 
            IF ( ICOMPQ .EQ. 1 ) THEN
*     Since QR won't update a Q matrix which is larger than the
*     length of D, we must solve the sub-problem in a workspace and
*     then multiply back into Q.
               CALL SSTEQR( 'I', M, D( START ), E( START ), 
     $              WORK, M, WORK( 1 + M*M ), INFO )
               CALL SLACPY( 'A', N, M, Q( 1, START ), LDQ,
     $              WORK( STOREQ ), N )
               CALL SGEMM( 'N', 'N', N, M, M, ONE, 
     $              WORK( STOREQ ), LDQ, WORK, M, ZERO, 
     $              Q( 1, START ) )
            ELSE IF (ICOMPQ .EQ. 2 ) THEN
               CALL SSTEQR( 'I', M, D( START ), E( START ), 
     $              Q( START, START ), LDQ, WORK, INFO )
            ELSE
               CALL SSTERF( M, D( START ), E( START ), INFO )
            END IF
            IF ( INFO .GT. 0 ) THEN
               INFO = START*(N+1) + END
               RETURN
            END IF
         END IF 
*
         START = END + 1
         GO TO 100
      END IF
*
*     endwhile
*
*     If the problem split any number of times, then the eigenvalues
*     will not be properly ordered.  Here we permute the eigenvalues
*     (and the associated eigenvectors) into ascending order.
*
      IF ( M .NE. N ) THEN
         IF (ICOMPQ .EQ. 0) THEN
*
*        Use Quick Sort
*
            CALL SLASRT( 'I', N, D, INFO )
*
         ELSE
*
*        Use Selection Sort to minimize swaps of eigenvectors
*
            DO 200  II = 2, N
               I = II - 1
               K = I
               P = D( I )
               DO 210  J = II, N
                  IF( D( J ) .LT. P ) THEN
                     K = J
                     P = D( J )
                  END IF
 210           CONTINUE
               IF( K .NE. I ) THEN
                  D( K ) = D( I )
                  D( I ) = P
                  CALL SSWAP( N, Q( 1, I ), 1, Q( 1, K ), 1 )
               END IF
 200        CONTINUE
         END IF
      END IF
*
      RETURN
*
*     End of SSTEDC
*
      END
